@@ -79,8 +79,7 @@
  * I moved this here because it seemed to be the most appropriate
  * place for it.  Define this if you want support for ``/window create''
  * and its related features.  If you dont want it, youll save some code,
- * and you wont need 'wserv', and if you do want it, you can have it in
- * all of its broken glory.
+ * and you wont need 'wserv'.
  */
 #define WINDOW_CREATE
 
@@ -307,6 +306,7 @@
 #define DEFAULT_KEY_INTERVAL 1000
 #define DEFAULT_LASTLOG 256
 #define DEFAULT_LASTLOG_LEVEL "ALL"
+#define DEFAULT_LASTLOG_REWRITE NULL
 #define DEFAULT_LOG 0
 #define DEFAULT_LOGFILE "irc.log"
 #define DEFAULT_MAIL 2
@@ -330,7 +330,6 @@
 #define DEFAULT_PAD_CHAR ' '
 #define DEFAULT_QUIT_MESSAGE "ircII %s -- Are we there yet?"
 #define DEFAULT_RANDOM_SOURCE 0
-#define DEFAULT_REVERSE_STATUS_LINE 1
 #define DEFAULT_SCREEN_OPTIONS NULL
 #define DEFAULT_SCROLLBACK 256
 #define DEFAULT_SCROLLBACK_RATIO 50
@@ -346,12 +345,13 @@
 #define DEFAULT_STATUS_CHANOP "@"
 #define DEFAULT_STATUS_CLOCK " %T"
 #define DEFAULT_STATUS_CPU_SAVER " (%L)"
-#define DEFAULT_STATUS_FORMAT "%T [%R] %*%=%@%N%#%S%H%B%Q%A%C%+%I%O%M%F%L %D %U %W"
-#define DEFAULT_STATUS_FORMAT1 "%T [%R] %*%=%@%N%#%S%H%B%Q%A%C%+%I%O%M%F%L %U "
-#define DEFAULT_STATUS_FORMAT2 "%W %X %Y %Z "
+#define DEFAULT_STATUS_FORMAT "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %D %U %W"
+#define DEFAULT_STATUS_FORMAT1 "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %U "
+#define DEFAULT_STATUS_FORMAT2 "%W %X %Y %Z "
 #define DEFAULT_STATUS_HALFOP "%"
 #define DEFAULT_STATUS_HOLD " Held: "
 #define DEFAULT_STATUS_HOLD_LINES "%B"
+#define DEFAULT_STATUS_HOLDMODE " (Hold) "
 #define DEFAULT_STATUS_INSERT ""
 #define DEFAULT_STATUS_MAIL " (Mail: %M)"
 #define DEFAULT_STATUS_MODE " (+%+)"
@@ -368,7 +368,7 @@
 #define DEFAULT_STATUS_SSL_ON "*SSL*"
 #define DEFAULT_STATUS_TRUNCATE_RHS 1
 #define DEFAULT_STATUS_UMODE " (+%#)"
-#define DEFAULT_STATUS_USER "EPIC5 -- Type /help for help"
+#define DEFAULT_STATUS_USER "EPIC5 -- Visit http://help.epicsol.org/ for help"
 #define DEFAULT_STATUS_USER1 ""
 #define DEFAULT_STATUS_USER2 ""
 #define DEFAULT_STATUS_USER3 ""
