@@ -10,10 +10,8 @@
 #define __input_h__
 
 /* miscelaneous functions */
-	void	edit_char			(unsigned char);
 	void	change_input_prompt 		(int);
 	void	cursor_to_input 		(void);
-	int	do_input_timeouts		(void *);
 	char *	get_input 			(void);
 	char *	get_input_prompt 		(void);
 	void	init_input 			(void);
@@ -21,7 +19,7 @@
 	char	input_pause 			(char *);
 	void	set_input 			(const char *);
 	void	set_input_prompt 		(void *);
-	void	update_input 			(int);
+	void	update_input 			(void *, int);
 
 /* keybinding functions */
 	BUILT_IN_KEYBINDING(backward_character);
@@ -61,13 +59,11 @@
 	BUILT_IN_KEYBINDING(toggle_insert_mode);
 	BUILT_IN_KEYBINDING(type_text);
 
-/* this was in keys.h, but it lives in input.c, so. */
-	BUILT_IN_COMMAND(typecmd);
-
 /* used by update_input */
 #define NO_UPDATE 0
 #define UPDATE_ALL 1
 #define UPDATE_FROM_CURSOR 2
 #define UPDATE_JUST_CURSOR 3
+#define CHECK_ZONES 4
 
 #endif /* _INPUT_H_ */
