@@ -7,7 +7,7 @@
  *
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  *
- * @(#)$Id: ircaux.h,v 1.89 2006/10/13 21:58:02 jnelson Exp $
+ * @(#)$Id: ircaux.h,v 1.98 2007/09/17 03:34:15 jnelson Exp $
  */
 
 #ifndef _IRCAUX_H_
@@ -83,7 +83,7 @@ FILE *	uzfopen 		(char **, const char *, int, struct stat *);
 int	end_strcmp 		(const char *, const char *, size_t);
 char*   exec_pipe		(const char *, char *, size_t *, char *const *);
 FILE **	open_exec		(const char *executable, char * const *args);
-void	panic 			(const char *, ...) __A(1) __N;
+void	panic 			(int, const char *, ...) __A(2) __N;
 int	fw_strcmp 		(comp_len_func *, char *, char *);
 int	lw_strcmp 		(comp_func *, char *, char *);
 int	open_to 		(char *, int, int);
@@ -108,6 +108,7 @@ char *	skip_spaces		(char *);
 int	split_args		(char *, char **to, size_t);
 int	splitw 			(char *, char ***, int);
 char *	unsplitw 		(char ***, int, int);
+int	split_string 		(char *, char ***, char);
 int	check_val 		(const char *);
 char *	strext	 		(const char *, const char *);
 char *	strextend 		(char *, char, int);
@@ -149,7 +150,7 @@ char *	strnrchr		(char *, char, int);
 void	mask_digits		(char **);
 char *	strpcat			(char *, const char *, ...) __A(2);
 char *  strlpcat		(char *, size_t, const char *, ...) __A(3);
-u_long	random_number		(u_long);
+unsigned long	random_number	(unsigned long);
 char *	urlencode		(const char *);
 char *	urldecode		(char *, size_t *);
 char *	enquote_it		(const char *str, size_t len);
@@ -181,6 +182,7 @@ char *	malloc_strdup 		(const char *);
 char *	malloc_strdup2 		(const char *, const char *);
 char *	malloc_strdup3 		(const char *, const char *, const char *);
 char *	malloc_strcat2_c	(char **, const char *, const char *, size_t *);
+char *	malloc_strcat3_c	(char **, const char *, const char *, const char *, size_t *);
 char *	malloc_strcat_wordlist_c (char **, const char *, const char *,size_t *);
 char *	malloc_strcat_word_c    (char **, const char *, const char *, int, size_t *);
 char *	malloc_sprintf 		(char **, const char *, ...) __A(2);
@@ -206,7 +208,7 @@ void    dequoter                (char **, size_t *, int, int, const char *);
 #define my_isdigit(x) \
 	(isdigit(*x) || ((*x == '-' || *x == '+') && isdigit(x[1])))
 
-int	vmy_strnicmp		(int, char *, ...);
+int	vmy_strnicmp		(size_t, char *, ...);
 char *	substitute_string	(const char *, const char *, const char *, int, int);
 
 /* ---------------- */
@@ -258,7 +260,11 @@ typedef struct Bucket Bucket;
 #define NUMSTR(x) (intmaxtoa((intmax_t)(x)))
 
 /********************/
+void	init_transforms (void);
 size_t	transform_string (int, int, const char *, const char *, size_t, char *, size_t);
-int	lookup_transform (const char *);
+int	lookup_transform (const char *, int *);
+char *	valid_transforms (void);
+
+extern int 	num_code_points (const char *);
 
 #endif /* _IRCAUX_H_ */
