@@ -30,6 +30,7 @@ extern int	LEVEL_QUIT;
 extern int	LEVEL_KICK;
 extern int	LEVEL_MODE;
 extern int	LEVEL_OPERWALL;
+extern int	LEVEL_SYSERR;
 extern int	LEVEL_USER1;
 extern int	LEVEL_USER2;
 extern int	LEVEL_USER3;
@@ -164,7 +165,7 @@ __inline static int	mask_isset (const Mask *set, int bit)
 	int		add_new_level	(const char *);
 	int		add_new_level_alias (int, const char *);
 	char *		get_all_levels	(void);
-	char *		mask_to_str	(const Mask *);
+	const char *	mask_to_str	(const Mask *);
 	int		str_to_mask	(Mask *, const char *);
 	const char *	level_to_str	(int);
 	int		str_to_level	(const char *);
