@@ -41,6 +41,8 @@ int	ALLOW_C1_CHARS_VAR,
 	DCC_STORE_PATH_VAR,
 	DCC_USE_GATEWAY_ADDR_VAR,
 	DEBUG_VAR,
+	DEFAULT_REALNAME_VAR,
+	DEFAULT_USERNAME_VAR,
 	DISPATCH_UNKNOWN_COMMANDS_VAR,
 	DISPLAY_VAR,
 	DO_NOTIFY_IMMEDIATELY_VAR,
@@ -63,6 +65,7 @@ int	ALLOW_C1_CHARS_VAR,
 	KEY_INTERVAL_VAR,
 	LASTLOG_VAR,
 	LASTLOG_LEVEL_VAR,
+	LASTLOG_REWRITE_VAR,
 	LOAD_PATH_VAR,
 	LOG_VAR,
 	LOGFILE_VAR,
@@ -92,8 +95,6 @@ int	ALLOW_C1_CHARS_VAR,
 	PAD_CHAR_VAR,
 	QUIT_MESSAGE_VAR,
 	RANDOM_SOURCE_VAR,
-	REALNAME_VAR,
-	REVERSE_STATUS_LINE_VAR,
 	SCREEN_OPTIONS_VAR,
 	SCROLLBACK_VAR,
 	SCROLLBACK_RATIO_VAR,
@@ -116,6 +117,7 @@ int	ALLOW_C1_CHARS_VAR,
 	STATUS_HALFOP_VAR,
 	STATUS_HOLD_VAR,
 	STATUS_HOLD_LINES_VAR,
+	STATUS_HOLDMODE_VAR,
 	STATUS_INSERT_VAR,
 	STATUS_MAIL_VAR,
 	STATUS_MODE_VAR,
@@ -214,7 +216,7 @@ typedef struct
         VARIABLE *      data;           /* The value of the variable */
         void    (*func) (void *); 	/* func called when var is set */
 	char *		script;		/* script called when var is set */
-        u_short         flags;          /* flags for this variable */
+        unsigned short  flags;          /* flags for this variable */
 }       IrcVariable;
 
 
