@@ -1,4 +1,4 @@
-/* $EPIC: commands.c,v 1.159 2006/11/04 17:37:34 jnelson Exp $ */
+/* $EPIC: commands.c,v 1.171 2007/09/07 18:07:29 jnelson Exp $ */
 /*
  * commands.c -- Stuff needed to execute commands in ircII.
  *		 Includes the bulk of the built in commands for ircII.
@@ -6,7 +6,7 @@
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1995, 2006 EPIC Software Labs
+ * Copyright  1995, 2007 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -104,8 +104,8 @@ static	void	commentcmd 	(const char *, char *, const char *);
 static	void	continuecmd	(const char *, char *, const char *);
 static	void	ctcp 		(const char *, char *, const char *);
 static	void	deop 		(const char *, char *, const char *);
-static	void	send_to_channel	(const char *, char *, const char *);
-static	void	send_to_target	(const char *, char *, const char *);
+static	void	send_to_channel_first	(const char *, char *, const char *);
+static	void	send_to_query_first	(const char *, char *, const char *);
 static	void	sendlinecmd 	(const char *, char *, const char *);
 static	void	echocmd		(const char *, char *, const char *);
 static	void	funny_stuff 	(const char *, char *, const char *);
@@ -144,10 +144,10 @@ static	void	send_comm 	(const char *, char *, const char *);
 static	void	send_invite 	(const char *, char *, const char *);
 static	void	send_kick 	(const char *, char *, const char *);
 static	void	send_channel_com (const char *, char *, const char *);
-static  void    set_username 	(const char *, char *, const char *);
 static	void	setenvcmd	(const char *, char *, const char *);
 static	void	squitcmd	(const char *, char *, const char *);
 static	void	subpackagecmd	(const char *, char *, const char *);
+static	void	typecmd 	(const char *, char *, const char *);
 static	void	usleepcmd	(const char *, char *, const char *);
 static  void	shift_cmd 	(const char *, char *, const char *);
 static	void	sleepcmd 	(const char *, char *, const char *);
@@ -187,7 +187,7 @@ const char *current_command = NULL;
  */
 static	IrcCommand irc_command[] =
 {
-	{ "",		send_to_channel	},
+	{ "",		send_to_channel_first	},
 	{ "#",		commentcmd	},
 	{ ":",		commentcmd	},
         { "ABORT",      abortcmd	},
@@ -235,7 +235,6 @@ static	IrcCommand irc_command[] =
 	{ "INPUT",	inputcmd	},
 	{ "INPUT_CHAR",	inputcmd	},
 	{ "INVITE",	send_invite	},
-	{ "IRCUSER",	set_username	},
 	{ "ISON",	isoncmd		},
 	{ "JOIN",	e_channel	},
 	{ "KICK",	send_kick	},
@@ -288,8 +287,8 @@ static	IrcCommand irc_command[] =
 #ifdef HAVE_RUBY
 	{ "RUBY",	rubycmd		}, /* ruby.c */
 #endif
-	{ "SAY",	send_to_channel	},
-	{ "SEND",	send_to_target	},
+	{ "SAY",	send_to_channel_first	},
+	{ "SEND",	send_to_query_first	},
 	{ "SENDLINE",	sendlinecmd	},
 	{ "SERVER",	servercmd	}, /* server.c */
 	{ "SERVLIST",	send_comm	},
@@ -621,7 +620,7 @@ BUILT_IN_COMMAND(describe)
 		say("Usage: /DESCRIBE <[=]nick|channel|*> <action description>");
 }
 
-BUILT_IN_COMMAND(send_to_target)
+BUILT_IN_COMMAND(send_to_query_first)
 {
 	const char	*tmp;
 
@@ -629,12 +628,14 @@ BUILT_IN_COMMAND(send_to_target)
 	send_text(from_server, tmp, args, NULL, 1);
 }
 
-BUILT_IN_COMMAND(send_to_channel)
+BUILT_IN_COMMAND(send_to_channel_first)
 {
 	const char	*tmp;
 
-	tmp = get_echannel_by_refnum(0);
-	send_text(from_server, tmp, args, NULL, 1);
+	if ((tmp = get_echannel_by_refnum(0)))
+	    send_text(from_server, tmp, args, NULL, 1);
+	else if ((tmp = get_target_by_refnum(0)))
+	    send_text(from_server, tmp, args, NULL, 1);
 }
 
 /*
@@ -852,6 +853,7 @@ BUILT_IN_COMMAND(xechocmd)
 	char	*flag_arg;
 	int	temp = 0;
 	int	all_windows = 0;
+	int	all_windows_for_server = 0;
 	int	want_banner = 0;
 	char	*stuff = NULL;
 	int	nolog = 0;
@@ -921,6 +923,18 @@ BUILT_IN_COMMAND(xechocmd)
 		     break;
 		}
 
+		case 'T':
+		{
+			/* Chew up the argument. */
+			next_arg(args, &args);
+
+			if (!(flag_arg = next_arg(args, &args)))
+				break;
+
+			to_from = flag_arg;
+			break;
+		}
+
 		case 'V':	/* VISUAL (output to a visible window) */
 		{
 			/* There is always at least one visible window! */
@@ -967,8 +981,14 @@ BUILT_IN_COMMAND(xechocmd)
 		case 'A':	/* ALL (output to all windows) */
 		case '*':
 		{
+			flag_arg = next_arg(args, &args);
+
+			if (toupper(flag_arg[2]) == 'S')
+				all_windows_for_server = 1;
+			else
+				all_windows = 1;
+
 			next_arg(args, &args);
-			all_windows = 1;
 			break;
 		}
 
@@ -1067,20 +1087,28 @@ BUILT_IN_COMMAND(xechocmd)
 		}
 	}
 	else if (want_banner != 0)
-		panic("xecho: want_banner is %d", want_banner);
+		panic(1, "xecho: want_banner is %d", want_banner);
 
-	if (all_windows == 1)
+	if (all_windows == 1 || all_windows_for_server == 1)
 	{
 		Window *win = NULL;
 		while ((traverse_all_windows(&win)))
 		{
-			int l = message_setall(win->refnum, to_from, to_level);
+			int	l;
+
+			if (all_windows == 0 && win->server != from_server)
+				continue;
+
+			l = message_setall(win->refnum, to_from, to_level);
 			put_echo(args);
 			pop_message_from(l);
 		}
 	}
 	else if (all_windows != 0)
-		panic("xecho: all_windows is %d", all_windows);
+		panic(1, "xecho: all_windows is %d", all_windows);
+	else if (all_windows_for_server != 0)
+		panic(1, "xecho: all_windows_for_server is %d", 
+			all_windows_for_server);
 	else
 	{
 		int l = message_setall(to_window_refnum, to_from, to_level);
@@ -1298,25 +1326,20 @@ BUILT_IN_COMMAND(info)
 		say("Versions 2.2pre8 through 2.8.2 by Matthew Green");
 		say("\tCopyright 1992-1995 Matthew Green");
 		say("All EPIC versions by EPIC Software Labs");
-		say("\tCopyright 1993-2006 EPIC Software Labs");
+		say("\tCopyright 1993-2007 EPIC Software Labs");
 		say(" ");
 		say("	    Contact the EPIC project (%s)", EMAIL_CONTACT);
 		say("	    for problems with this or any other EPIC client");
 		say(" ");
 		say("EPIC Software Labs (in alphabetical order):");
-		say("       \tB. Thomas Frazier    <ay-ar@epicsol.org>");
 		say("       \tBrian Hauber         <bhauber@epicsol.org>");
 		say("       \tChip Norkus          <wd@epicsol.org>");
 		say("       \tCrazyEddy            <crazyed@epicsol.org>");
 		say("	    \tDennis Moore         <nimh@epicsol.org>");
                 say("       \tErlend B. Mikkelsen  <howl@epicsol.org>");
-		say("	    \tJake Khuon           <khuon@epicsol.org>");
 		say("       \tJason Brand          <kitambi@epicsol.org>");
 		say("	    \tJeremy Nelson        <jnelson@epicsol.org>");
-		say("       \tRobert Chady         <chady@epicsol.org>");
-		say("       \tRobohak              <robohak@epicsol.org>");
-		say("       \tSrfRoG               <cag@epicsol.org>");
-		say("       \tTerry Warner         <keerf@epicsol.org>");
+		say("       \tStanislaw Halik      <sthalik@epicsol.org>");
 		say("       \tWilliam Rockwood     <wjr@epicsol.org>");
                 say("       \tXavier               <jak@epicsol.org>");
 		say(" ");
@@ -1404,7 +1427,7 @@ BUILT_IN_COMMAND(license)
 	yell("Copyright (c) 1991, 1992 Troy Rollo.");
  	yell("Copyright (c) 1992-1996 Matthew Green.");
 	yell("Copyright  1994 Jake Khuon.");
-	yell("Coypright  1993, 2006 EPIC Software Labs.");
+	yell("Coypright  1993, 2007 EPIC Software Labs.");
 	yell("All rights reserved");
 	yell(" ");
 	yell("Redistribution and use in source and binary forms, with or");
@@ -2396,7 +2419,8 @@ BUILT_IN_COMMAND(send_invite)
 	    else
 		say("You are not on a channel");
 	}
-	else
+
+	if (!invites)
 	    say("Usage: %s [#channel] nickname", command);
 }
 
@@ -2469,11 +2493,12 @@ BUILT_IN_COMMAND(sendlinecmd)
 	display = window_display;
 	window_display = 1;
 	parse_statement(args, 1, NULL);
-	update_input(UPDATE_ALL);
+	update_input(NULL, UPDATE_ALL);
 	window_display = display;
 	from_server = server;
 }
 
+#if 0
 /* 
  * IRCUSER command. Changes your userhost on the fly.  Takes effect
  * the next time you connect to a server 
@@ -2494,6 +2519,7 @@ BUILT_IN_COMMAND(set_username)
 		say ("Usage: /IRCUSER <text>");
 #endif
 }
+#endif
 
 BUILT_IN_COMMAND(setenvcmd)
 {
@@ -2765,6 +2791,58 @@ BUILT_IN_COMMAND(whois)
 					args);
 }
 
+/*
+ * type: The TYPE command.  This parses the given string and treats each
+ * character as though it were typed in by the user.  Thus key bindings
+ * are used for each character parsed.  Special case characters are control
+ * character sequences, specified by a ^ follow by a legal control key.
+ * Thus doing "/TYPE ^B" will be as tho ^B were hit at the keyboard,
+ * probably moving the cursor backward one character.
+ *
+ * This was moved from keys.c, because it certainly does not belong there,
+ * and this seemed a reasonable place for it to go for now.
+ */
+BUILT_IN_COMMAND(typecmd)
+{
+        int     c;
+        char    key;
+
+        for (; *args; args++)
+        {
+                if (*args == '^')
+                {
+                        args++;
+                        if (*args == '?')
+                                key = '\177';
+                        else if (*args)
+                        {
+                                c = *args;
+                                if (islower(c))
+                                        c = toupper(c);
+                                if (c < 64)
+                                {
+                                        say("Invalid key sequence: ^%c", c);
+                                        return;
+                                }
+                                key = c - 64;
+                        }
+                        else
+                                break;
+                }
+                else
+                {
+                        if (*args == '\\')
+                                args++;
+                        if (*args)
+                                key = *args;
+                        else
+                                break;
+                }
+
+                edit_char(key);
+        }
+}
+
 BUILT_IN_COMMAND(xtypecmd)
 {
 	char	*arg;
@@ -3048,7 +3126,8 @@ struct target_type target[4] =
 		}
 
 		i = is_channel(current_nick);
-		if (get_server_doing_privmsg(from_server) || (command && !strcmp(command, "NOTICE")))
+		if (get_server_doing_privmsg(from_server) || 
+				(command && !strcmp(command, "NOTICE")))
 			i += 2;
 
 		if ((key = is_crypted(current_nick, from_server, ANYCRYPT)))
@@ -3405,7 +3484,7 @@ static void	parse_block (const char *org_line, const char *args, int interactive
 	 * We will be mangling 'org_line', so we make a copy to work with.
 	 */
 	if (!org_line)
-		panic("org_line is NULL and it shouldn't be.");
+		panic(1, "org_line is NULL and it shouldn't be.");
 	line = LOCAL_COPY(org_line);
 
 	/*
@@ -3596,7 +3675,7 @@ static	unsigned 	level = 0;
 		const char *alias = NULL;
 		void	*arglist = NULL;
 		void	(*builtin) (const char *, char *, const char *) = NULL;
-		const char *prevcmd;
+		const char *prevcmd = NULL;
 
 		if (subargs != NULL)
 			cmd = expand_alias(stmt, subargs); 
