@@ -1,4 +1,4 @@
-/* $EPIC: compat.c,v 1.28 2006/06/07 02:41:54 jnelson Exp $ */
+/* $EPIC: compat.c,v 1.32 2007/08/30 03:29:40 jnelson Exp $ */
 /*
  * Everything that im not directly responsible for I put in here.  Almost
  * all of this stuff is either borrowed from somewhere else (for you poor
@@ -1005,9 +1005,9 @@ size_t	strlcat (char *dst, const char *src, size_t siz)
  * RC4 is a registered trademark of RSA Laboratories.
  */
 struct bsd_arc4_stream {
-	u_char	i;
-	u_char	j;
-	u_char	s[256];
+	unsigned char	i;
+	unsigned char	j;
+	unsigned char	s[256];
 };
 typedef struct bsd_arc4_stream 	ARC4;
 
@@ -1026,10 +1026,10 @@ static void	bsd_arc4_init (ARC4 *as)
 }
 
 __inline__
-static void	bsd_arc4_addrandom (ARC4 *as, u_char *dat, int datlen)
+static void	bsd_arc4_addrandom (ARC4 *as, unsigned char *dat, int datlen)
 {
 	int     n;
-	u_char	si;
+	unsigned char	si;
 
 	as->i--;
 	for (n = 0; n < 256; n++) {
@@ -1047,7 +1047,7 @@ static void	bsd_arc4_stir (ARC4 *as)
 	struct {
 		Timeval tv;
 		pid_t 	pid;
-		u_char	rnd[128 - sizeof(Timeval) - sizeof(pid_t)];
+		unsigned char	rnd[128 - sizeof(Timeval) - sizeof(pid_t)];
 	}       rdat;
 
 	gettimeofday(&rdat.tv, NULL);
@@ -1064,9 +1064,9 @@ static void	bsd_arc4_stir (ARC4 *as)
 }
 
 __inline__
-static u_char		bsd_arc4_getbyte (ARC4 *as)
+static unsigned char		bsd_arc4_getbyte (ARC4 *as)
 {
-	u_char si, sj;
+	unsigned char si, sj;
 
 	as->i = (as->i + 1);
 	si = as->s[as->i];
@@ -1098,7 +1098,7 @@ void	bsd_arc4random_stir (void)
 	bsd_arc4_stir(&rs);
 }
 
-void	bsd_arc4random_addrandom (u_char *dat, int datlen)
+void	bsd_arc4random_addrandom (unsigned char *dat, int datlen)
 {
 	if (!rs_initialized)
 		bsd_arc4random_stir();
@@ -1142,7 +1142,7 @@ int	my_base64_encode (const void *data, int size, char **str)
 {
     char *s, *p;
     int i;
-    int c;
+    unsigned c;
     const unsigned char *q;
 
     p = s = (char *)new_malloc(size * 4 / 3 + 4);
@@ -1151,14 +1151,14 @@ int	my_base64_encode (const void *data, int size, char **str)
     q = (const unsigned char *) data;
     i = 0;
     for (i = 0; i < size;) {
-	c = q[i++];
+	c = (unsigned)(unsigned char)q[i++];
 	c *= 256;
 	if (i < size)
-	    c += q[i];
+	    c += (unsigned)(unsigned char)q[i];
 	i++;
 	c *= 256;
 	if (i < size)
-	    c += q[i];
+	    c += (unsigned)(unsigned char)q[i];
 	i++;
 	p[0] = base64_chars[(c & 0x00fc0000) >> 18];
 	p[1] = base64_chars[(c & 0x0003f000) >> 12];
@@ -1355,10 +1355,10 @@ int vsnprintf (char *str, size_t size, const char *format, va_list ap)
 	/* If the string ended up overflowing, just give up. */
 	/* Pre-ansi vsprintf()s return (char *) */
 	if (ret == (int)str && strlen(str) > size)
-		panic("Buffer overflow in vsnprintf");
+		panic(1, "Buffer overflow in vsnprintf");
 	/* ANSI vsprintf()s return (int) */
 	if (ret != (int)str && ret > size)
-		panic("Buffer overflow in vsnprintf");
+		panic(1, "Buffer overflow in vsnprintf");
 
 	/* We always return (int). */
 	return ret;
@@ -1456,5 +1456,12 @@ long long	strtoll (const char *nptr, char **endptr, int base)
 }
 #endif
 
+#ifndef HAVE_STRTOIMAX
+long		strtoimax (const char *nptr, char **endptr, int base)
+{
+	return (long)strtol(nptr, endptr, base);
+}
+#endif
+
 /**** END MISC PUBLIC DOMAIN STUFF ****/
 
