@@ -1,4 +1,4 @@
-/* $EPIC: expr2.c,v 1.35 2006/10/13 21:58:02 jnelson Exp $ */
+/* $EPIC: expr2.c,v 1.37 2007/06/01 01:39:31 jnelson Exp $ */
 /*
  * Zsh: math.c,v 3.1.2.1 1997/06/01 06:13:15 hzoli Exp 
  * math.c - mathematical expression evaluation
@@ -321,7 +321,7 @@ enum LEX {
 	M_FUNCTION,	M_INPAR,
 	NOT, 		COMP, 		PREMINUS,	PREPLUS,
 			UPLUS,		UMINUS,		STRLEN,
-			WORDC,		DEREF,
+			WORDC,		DEREF,		REEXP,
 	POWER,
 	MUL,		DIV,		MOD,
 	PLUS,		MINUS,		STRCAT,
@@ -370,7 +370,7 @@ static	int	prec[TOKCOUNT] =
 	1,		1,
 	2,		2,		2,		2,
 			2,		2,		2,
-			2,		2,
+			2,		2,		2,
 	3,
 	4,		4,		4,
 	5,		5,		5,
@@ -424,7 +424,7 @@ static 	int 	assoc[TOKCOUNT] =
 	LR,		LR,
 	RL,		RL,		RL,		RL,
 			RL,		RL,		RL,
-			RL,		RL,
+			RL,		RL,		RL,
 	RL,
 	LR,		LR,		LR,
 	LR,		LR,		LR,
@@ -608,7 +608,7 @@ __inline static	const char *	get_token_raw (expr_info *c, TOKEN v)
 		TOK(c, v).used |= USED_RAW;
 
 		if (TOK(c, v).used & USED_EXPANDED)
-			panic("Cannot convert EXPANDED token to RAW");
+			panic(1, "Cannot convert EXPANDED token to RAW");
 		else if (TOK(c, v).used & USED_FLOAT)
 		{
 			TOK(c, v).raw_value = 
@@ -637,7 +637,7 @@ __inline static	const char *	get_token_raw (expr_info *c, TOKEN v)
 					v, TOK(c, v).lval, TOK(c, v).raw_value);
 		}
 		else
-			panic("Can't convert this token to raw format");
+			panic(1, "Can't convert this token to raw format");
 	}
 
 	return TOK(c, v).raw_value;
@@ -750,7 +750,7 @@ __inline static	const char *	get_token_expanded (expr_info *c, TOKEN v)
 					v, myval, TOK(c, v).expanded_value);
 		}
 		else
-			panic("Cannot convert from this token to EXPANDED");
+			panic(1, "Cannot convert from this token to EXPANDED");
 	}
 
 	if (x_debug & DEBUG_NEW_MATH_DEBUG)
@@ -1385,6 +1385,20 @@ static void	reduce (expr_info *cx, int what)
 			push_lval(cx, s);
 			break;
 		}
+		case REEXP:
+		{
+			const char *val;
+
+			s = pop_expanded(cx);
+			if (cx->noeval)
+				cx->last_token = 0;
+			else
+				cx->last_token = tokenize_raw(cx, s);
+
+			push_token(cx, cx->last_token);
+			break;
+		}
+
 
 		/* (pre|post)(in|de)crement operators. */
 		case PREPLUS:   AUTO_UNARY(j + 1, j + 1)
@@ -1819,6 +1833,11 @@ static int	zzlex (expr_info *c)
 				c->ptr++;
 				if (*c->ptr == '=') 
 					OPERATOR("**=", 1, POWEREQ)
+				else if (c->operand)
+				{
+					dummy = 2;
+					UNARY("**", 0, REEXP)
+				}
 				else
 					OPERATOR("**", 0, POWER)
 			}
