@@ -1,4 +1,4 @@
-/* $EPIC: hook.c,v 1.69 2006/10/13 21:58:02 jnelson Exp $ */
+/* $EPIC: hook.c,v 1.77 2007/09/07 18:07:29 jnelson Exp $ */
 /*
  * hook.c: Does those naughty hook functions. 
  *
@@ -134,105 +134,106 @@ typedef struct Hookables
 	int	mark;			/* Hook type is currently active */
 	unsigned flags;			/* Anything else needed */
 	char *	implied;		/* Implied output if unhooked */
+	int	implied_protect;	/* Do not re-expand implied hook */
 } Hookables;
 
 Hookables hook_function_templates[] =
 {
-	{ "ACTION",		NULL,	3,	0,	0,	NULL },
-	{ "CHANNEL_LOST", 	NULL, 	2,  	0,  	0,  	NULL },
-	{ "CHANNEL_NICK",	NULL,	3,	0,	0,	NULL },
-	{ "CHANNEL_SIGNOFF",	NULL,	3,	0,	0,	NULL },
-	{ "CHANNEL_SYNC",	NULL,	3,	0,	0,	NULL },
-	{ "CONNECT",		NULL,	2,	0,	0,	NULL },
-	{ "CTCP",		NULL,	4,	0,	0,	NULL },
-	{ "CTCP_REPLY",		NULL,	4,	0,	0,	NULL },
-	{ "CTCP_REQUEST",	NULL,	4,	0,	0,	NULL },
-	{ "DCC_ACTIVITY",	NULL,	1,	0,	0,	NULL },
-	{ "DCC_CHAT",		NULL,	2,	0,	0,	NULL },
-        { "DCC_CONNECT",        NULL,   2,      0,      0,	NULL },
-	{ "DCC_LIST",		NULL,	8,	0,	0,	NULL },
-        { "DCC_LOST",           NULL,   2,      0,      0,	NULL },
-	{ "DCC_OFFER", 		NULL,	2,	0,	0,	NULL },
-	{ "DCC_RAW",		NULL,	3,	0,	0,	NULL },
-        { "DCC_REQUEST",        NULL,   4,      0,      0,	NULL },
-	{ "DISCONNECT",		NULL,	1,	0,	0,	NULL },
-        { "ENCRYPTED_NOTICE",   NULL,   3,      0,      0,	NULL },
-        { "ENCRYPTED_PRIVMSG",  NULL,   3,      0,      0,	NULL },
-	{ "ERROR",		NULL,	1,	0,	0,	NULL },
-	{ "EXEC",		NULL,	2,	0,	0,	NULL },
-	{ "EXEC_ERRORS",	NULL,	2,	0,	0,	NULL },
-	{ "EXEC_EXIT",		NULL,	3,	0,	0,	NULL },
-	{ "EXEC_PROMPT",	NULL,	2,	0,	0,	NULL },
-        { "EXIT",               NULL,   1,      0,      0,	NULL },
-	{ "FLOOD",		NULL,	5,	0,	0,	NULL },
-	{ "GENERAL_NOTICE",	NULL,	3,	0,	0,	NULL },
-	{ "GENERAL_PRIVMSG",	NULL,	3,	0,	0,	NULL },
-	{ "HELP",		NULL,	2,	0,	0,	NULL },
-	{ "HOOK",		NULL,	1,	0,	0,	NULL },
-	{ "IDLE",		NULL,	1,	0,	0,	NULL },
-	{ "INPUT",		NULL,	1,	0,	HF_NORECURSE,	NULL },
-	{ "INVITE",		NULL,	3,	0,	0,	NULL },
-	{ "JOIN",		NULL,	4,	0,	0,	NULL },
-	{ "KEYBINDING",		NULL,	3,	0,	0,	NULL },
-	{ "KICK",		NULL,	3,	0,	0,	NULL },
-	{ "KILL",		NULL,	5,	0,	0,	NULL },
-	{ "LIST",		NULL,	3,	0,	0,	NULL },
-	{ "MAIL",		NULL,	2,	0,	0,	NULL },
-	{ "MODE",		NULL,	3,	0,	0,	NULL },
-	{ "MODE_STRIPPED",	NULL,	3,	0,	0,	NULL },
-	{ "MSG",		NULL,	2,	0,	0,	NULL },
-	{ "MSG_GROUP",		NULL,	3,	0,	0,	NULL },
-	{ "NAMES",		NULL,	2,	0,	0,	NULL },
-	{ "NEW_NICKNAME",	NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "NICKNAME",		NULL,	2,	0,	0,	NULL },
-	{ "NOTE",		NULL,	3,	0,	0,	NULL },
-	{ "NOTICE",		NULL,	2,	0,	0,	NULL },
-	{ "NOTIFY_SIGNOFF",	NULL,	1,	0,	0,	NULL },
-	{ "NOTIFY_SIGNON",	NULL,	2,	0,	0,	NULL },
-	{ "NUMERIC",		NULL,	3,	0,	0,	NULL },
-	{ "ODD_SERVER_STUFF",	NULL,	3,	0,	0,	NULL },
-	{ "OPERWALL",		NULL,	2,	0,	0,	NULL },
-	{ "OPER_NOTICE",	NULL,	2,	0,	0,	NULL },
-	{ "PART",		NULL,	2,	0,	0,	NULL },
-	{ "PONG",		NULL,	2,	0,	0,	NULL },
-	{ "PUBLIC",		NULL,	3,	0,	0,	NULL },
-	{ "PUBLIC_MSG",		NULL,	3,	0,	0,	NULL },
-	{ "PUBLIC_NOTICE",	NULL,	3,	0,	0,	NULL },
-	{ "PUBLIC_OTHER",	NULL,	3,	0,	0,	NULL },
-	{ "RAW_IRC",		NULL,	1,	0,	0,	NULL },
-	{ "REDIRECT",		NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "SEND_ACTION",	NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "SEND_CTCP",		NULL,	3,	0,	HF_NORECURSE,	NULL },
-	{ "SEND_DCC_CHAT",	NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "SEND_MSG",		NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "SEND_NOTICE",	NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "SEND_PUBLIC",	NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "SEND_TO_SERVER",	NULL,	3,	0,	HF_NORECURSE,	NULL },
-	{ "SERVER_ESTABLISHED",	NULL,	2,	0,	0,	NULL },
-	{ "SERVER_LOST",	NULL,	2,	0,	0,	NULL },
-	{ "SERVER_NOTICE",	NULL,	1,	0,	0,	NULL },
-	{ "SERVER_STATUS",	NULL,	3,	0,	0,	NULL },
-	{ "SET",		NULL,	2,	0,	0,	NULL },
-	{ "SIGNOFF",		NULL,	1,	0,	0,	NULL },
-	{ "SILENCE",		NULL,	2,	0,	0,	NULL },
-	{ "SSL_SERVER_CERT",	NULL,	3,	0,	0,	NULL },
-	{ "STATUS_UPDATE",	NULL,	3,	0,	0,	NULL },
-	{ "SWITCH_CHANNELS",	NULL,	3,	0,	0,	NULL },
-	{ "SWITCH_WINDOWS",	NULL,	4,	0,	0,	NULL },
-	{ "TIMER",		NULL,	1,	0,	0,	NULL },
-	{ "TOPIC",		NULL,	2,	0,	0,	NULL },
-	{ "UNKNOWN_COMMAND",	NULL,	2,	0,	HF_NORECURSE, 	NULL},
-	{ "UNLOAD",		NULL,	1,	0,	0,	NULL },
-	{ "WALL",		NULL,	2,	0,	0,	NULL },
-	{ "WALLOP",		NULL,	3,	0,	0,	NULL },
-	{ "WHO",		NULL,	6,	0,	0,	NULL },
-	{ "WINDOW",		NULL,	2,	0,	HF_NORECURSE,	NULL },
-	{ "WINDOW_COMMAND",	NULL,	1, 	0,	0,	NULL },
-	{ "WINDOW_CREATE",	NULL,	1, 	0,	0,	NULL },
-	{ "WINDOW_BEFOREKILL",	NULL,	1,	0,	0,	NULL },
-	{ "WINDOW_KILL",	NULL,	2,	0,	0,	NULL },
-	{ "WINDOW_SERVER",	NULL,	3,	0,	0,	NULL },
-	{ "YELL",		NULL,	1,	0,	0,	NULL },
+	{ "ACTION",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "CHANNEL_LOST", 	NULL, 	2,  	0,  	0,  	NULL, 0 },
+	{ "CHANNEL_NICK",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "CHANNEL_SIGNOFF",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "CHANNEL_SYNC",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "CONNECT",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "CTCP",		NULL,	4,	0,	0,	NULL, 0 },
+	{ "CTCP_REPLY",		NULL,	4,	0,	0,	NULL, 0 },
+	{ "CTCP_REQUEST",	NULL,	4,	0,	0,	NULL, 0 },
+	{ "DCC_ACTIVITY",	NULL,	1,	0,	0,	NULL, 0 },
+	{ "DCC_CHAT",		NULL,	2,	0,	0,	NULL, 0 },
+        { "DCC_CONNECT",        NULL,   2,      0,      0,	NULL, 0 },
+	{ "DCC_LIST",		NULL,	8,	0,	0,	NULL, 0 },
+        { "DCC_LOST",           NULL,   2,      0,      0,	NULL, 0 },
+	{ "DCC_OFFER", 		NULL,	2,	0,	0,	NULL, 0 },
+	{ "DCC_RAW",		NULL,	3,	0,	0,	NULL, 0 },
+        { "DCC_REQUEST",        NULL,   4,      0,      0,	NULL, 0 },
+	{ "DISCONNECT",		NULL,	1,	0,	0,	NULL, 0 },
+        { "ENCRYPTED_NOTICE",   NULL,   3,      0,      0,	NULL, 0 },
+        { "ENCRYPTED_PRIVMSG",  NULL,   3,      0,      0,	NULL, 0 },
+	{ "ERROR",		NULL,	1,	0,	0,	NULL, 0 },
+	{ "EXEC",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "EXEC_ERRORS",	NULL,	2,	0,	0,	NULL, 0 },
+	{ "EXEC_EXIT",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "EXEC_PROMPT",	NULL,	2,	0,	0,	NULL, 0 },
+        { "EXIT",               NULL,   1,      0,      0,	NULL, 0 },
+	{ "FLOOD",		NULL,	5,	0,	0,	NULL, 0 },
+	{ "GENERAL_NOTICE",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "GENERAL_PRIVMSG",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "HELP",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "HOOK",		NULL,	1,	0,	0,	NULL, 0 },
+	{ "IDLE",		NULL,	1,	0,	0,	NULL, 0 },
+	{ "INPUT",		NULL,	1,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "INVITE",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "JOIN",		NULL,	4,	0,	0,	NULL, 0 },
+	{ "KEYBINDING",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "KICK",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "KILL",		NULL,	5,	0,	0,	NULL, 0 },
+	{ "LIST",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "MAIL",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "MODE",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "MODE_STRIPPED",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "MSG",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "MSG_GROUP",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "NAMES",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "NEW_NICKNAME",	NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "NICKNAME",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "NOTE",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "NOTICE",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "NOTIFY_SIGNOFF",	NULL,	1,	0,	0,	NULL, 0 },
+	{ "NOTIFY_SIGNON",	NULL,	2,	0,	0,	NULL, 0 },
+	{ "NUMERIC",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "ODD_SERVER_STUFF",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "OPERWALL",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "OPER_NOTICE",	NULL,	2,	0,	0,	NULL, 0 },
+	{ "PART",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "PONG",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "PUBLIC",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "PUBLIC_MSG",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "PUBLIC_NOTICE",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "PUBLIC_OTHER",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "RAW_IRC",		NULL,	1,	0,	0,	NULL, 0 },
+	{ "REDIRECT",		NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SEND_ACTION",	NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SEND_CTCP",		NULL,	3,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SEND_DCC_CHAT",	NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SEND_MSG",		NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SEND_NOTICE",	NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SEND_PUBLIC",	NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SEND_TO_SERVER",	NULL,	3,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "SERVER_ESTABLISHED",	NULL,	2,	0,	0,	NULL, 0 },
+	{ "SERVER_LOST",	NULL,	2,	0,	0,	NULL, 0 },
+	{ "SERVER_NOTICE",	NULL,	1,	0,	0,	NULL, 0 },
+	{ "SERVER_STATUS",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "SET",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "SIGNOFF",		NULL,	1,	0,	0,	NULL, 0 },
+	{ "SILENCE",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "SSL_SERVER_CERT",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "STATUS_UPDATE",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "SWITCH_CHANNELS",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "SWITCH_WINDOWS",	NULL,	4,	0,	0,	NULL, 0 },
+	{ "TIMER",		NULL,	1,	0,	0,	NULL, 0 },
+	{ "TOPIC",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "UNKNOWN_COMMAND",	NULL,	2,	0,	HF_NORECURSE, 	NULL, 0},
+	{ "UNLOAD",		NULL,	1,	0,	0,	NULL, 0 },
+	{ "WALL",		NULL,	2,	0,	0,	NULL, 0 },
+	{ "WALLOP",		NULL,	3,	0,	0,	NULL, 0 },
+	{ "WHO",		NULL,	6,	0,	0,	NULL, 0 },
+	{ "WINDOW",		NULL,	2,	0,	HF_NORECURSE,	NULL, 0 },
+	{ "WINDOW_COMMAND",	NULL,	1, 	0,	0,	NULL, 0 },
+	{ "WINDOW_CREATE",	NULL,	1, 	0,	0,	NULL, 0 },
+	{ "WINDOW_BEFOREKILL",	NULL,	1,	0,	0,	NULL, 0 },
+	{ "WINDOW_KILL",	NULL,	2,	0,	0,	NULL, 0 },
+	{ "WINDOW_SERVER",	NULL,	3,	0,	0,	NULL, 0 },
+	{ "YELL",		NULL,	1,	0,	0,	NULL, 0 },
 };
 
 static Hookables *hook_functions = NULL;
@@ -273,6 +274,7 @@ static void	initialize_hook_functions (void)
 		hook_functions[i].mark = 0;
 		hook_functions[i].flags = 0;
 		hook_functions[i].implied = NULL;
+		hook_functions[i].implied_protect = 0;
 	}
 
 	for (b = 0, i = FIRST_NAMED_HOOK; i < NUMBER_OF_LISTS; b++, i++)
@@ -283,6 +285,7 @@ static void	initialize_hook_functions (void)
 		hook_functions[i].mark = hook_function_templates[b].mark;
 		hook_functions[i].flags = hook_function_templates[b].flags;
 		hook_functions[i].implied = NULL;
+		hook_functions[i].implied_protect = 0;
 	}
 
 	if (hooklist == NULL)
@@ -699,26 +702,33 @@ int 	do_hook (int which, const char *format, ...)
 	char		quote;
 	int		serial_number;
 	struct Current_hook *hook;
+	Hookables *	h;
 
 	*buffer = 0;
 
 	if (!hook_functions_initialized)
 		initialize_hook_functions();
+	h = &hook_functions[which];
 
-	/* If deny_all_hooks is set, no hooks are permitted executed. */
-	if (deny_all_hooks || !hook_functions[which].list)
+	/*
+	 * Decide whether to post this event.  Events are suppressed if:
+	 *   1) $hookctl(DENY_ALL_HOOKS 1) has been turned on
+	 *   2) There are no /on's and no implied hooks
+	 *   3) The /on has recursed and that is forbidden.
+	 */
+	if (deny_all_hooks || 
+	    (!h->list && !h->implied) ||
+	    (h->mark && h->flags & HF_NORECURSE))
 	{
 		retval = NO_ACTION_TAKEN;
-		goto implied_hook;
+		return retval;
 	}
 
 	/*
-	 * If we're already executing the type, and we're
-	 * specifically not supposed to allow recursion, then
-	 * dont allow recursion. ;-)
+	 * If there are no /on's, but there is an implied hook, skip
+	 * right to the implied hook.
 	 */
-	if (hook_functions[which].mark && 
-	    (hook_functions[which].flags & HF_NORECURSE))
+	if (!h->list && h->implied)
 	{
 		retval = NO_ACTION_TAKEN;
 		goto implied_hook;
@@ -738,7 +748,7 @@ int 	do_hook (int which, const char *format, ...)
 		va_end(args);
 	}
 	else
-		panic("do_hook: format is NULL");
+		panic(1, "do_hook: format is NULL");
 
 	/*
 	 * Set current_hook
@@ -757,14 +767,12 @@ int 	do_hook (int which, const char *format, ...)
  	 * unwanted recursion in some /on's.
 	 */
 	if (which >= 0)
-		hook_functions[which].mark++;
+		h->mark++;
 
         serial_number = INT_MIN;
         for (;!hook->halt;serial_number++)
 	{
-	    for (tmp = hook_functions[which].list; 
-			!hook->halt && tmp;
-			tmp = tmp->next)
+	    for (tmp = h->list; !hook->halt && tmp; tmp = tmp->next)
             {
 		Hook *besthook = NULL;
 		ArgList *tmp_arglist;
@@ -817,7 +825,7 @@ int 	do_hook (int which, const char *format, ...)
 		/* Copy off everything important from 'tmp'. */
 		noise = tmp->noisy;
 		if (!name)
-			name = LOCAL_COPY(hook_functions[which].name);
+			name = LOCAL_COPY(h->name);
 		stuff_copy = LOCAL_COPY(tmp->stuff);
 		quote = tmp->flexible ? '\'' : '"';
 
@@ -910,7 +918,7 @@ int 	do_hook (int which, const char *format, ...)
 	 * Mark the event as not currently being done here.
 	 */
 	if (which >= 0)
-		hook_functions[which].mark--;
+		h->mark--;
 
 	/*
 	 * Reset current_hook to it's previous value.
@@ -924,7 +932,7 @@ int 	do_hook (int which, const char *format, ...)
 	/*
 	 * And return the user-specified suppression level
 	 */
-	if (retval == SUPPRESS_DEFAULT || !hook_functions[which].implied)
+	if (retval == SUPPRESS_DEFAULT || !h->implied)
 		return retval;
 
     implied_hook:
@@ -934,7 +942,7 @@ int 	do_hook (int which, const char *format, ...)
 	char *func_call = NULL;
 	char *func_retval;
 
-	if (!hook_functions[which].implied)
+	if (!h->implied)
 		break;
 
 	if (*buffer == 0)
@@ -947,17 +955,34 @@ int 	do_hook (int which, const char *format, ...)
 		va_end(args);
 	    }
 	    else
-		panic("do_hook: format is NULL");
+		panic(1, "do_hook: format is NULL");
 	}
 
-	malloc_sprintf(&func_call, "cparse(\"%s\" $*)", 
-			hook_functions[which].implied);
+	if (which >= 0)
+		h->mark++;
+
+	if (h->implied_protect)
+	{
+	    malloc_sprintf(&func_call, "cparse(\"$*\" %s)", buffer);
+	    func_retval = call_function(func_call, h->implied);
+	}
+	else
+	{
+	    malloc_sprintf(&func_call, "cparse(\"%s\" $*)", h->implied);
+	    func_retval = call_function(func_call, buffer);
+	}
+
+/*
 	func_retval = call_function(func_call, buffer);
+*/
 	put_echo(func_retval);
 
 	new_free(&func_call);
 	new_free(&func_retval);
 	retval = SUPPRESS_DEFAULT;
+
+	if (which >= 0)
+		h->mark--;
     }
     while (0);
 #endif
@@ -1996,8 +2021,7 @@ char *hookctl (char *input)
 		else
 		{
 			GET_FUNC_ARG(str,input);
-			prop = vmy_strnicmp(strlen(str), str, "LISTS", "POPULATED_LISTS",
-				"HOOKS");
+			prop = vmy_strnicmp(strlen(str), str, "LISTS", "POPULATED_LISTS", "HOOKS", NULL);
 		}
 		if (input && *input)
 		{
@@ -2526,6 +2550,7 @@ char *hookctl (char *input)
 										input[span + 1] = 0;
 										input++;
 									}
+									hooks->implied_protect = 1;
 								}
 								if (empty(input))
 									new_free(&hooks->implied);
@@ -2694,8 +2719,8 @@ char *hookctl (char *input)
 			if (!hook)
 				break;
 		}
-		RETURN_MSTR(ret);
 		new_free (&buffer);
+		RETURN_MSTR(ret);
 		break;
 
 	/* go-switch */
