@@ -1,4 +1,4 @@
-/* $EPIC: irc.c,v 1.1102 2006/11/11 15:47:19 jnelson Exp $ */
+/* $EPIC: irc.c,v 1.1181 2007/09/18 02:34:26 jnelson Exp $ */
 /*
  * ircII: a new irc client.  I like it.  I hope you will too!
  *
@@ -6,7 +6,7 @@
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
  * Copyright  1994 Jake Khuon.
- * Copyright  1993, 2006 EPIC Software Labs.
+ * Copyright  1993, 2007 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,19 +40,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC5-0.3.2";
-const char useful_info[] = "epic5 0 3 2";
+const char irc_version[] = "EPIC5-0.3.5";
+const char useful_info[] = "epic5 0 3 5";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20061111";
+const char internal_version[] = "20070917";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 1431;
+const unsigned long	commit_id = 1509;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -60,7 +60,7 @@ const unsigned long	commit_id = 1431;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Mitigation";
+const char ridiculous_version_name[] = "Alacrity";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -99,12 +99,6 @@ const char ridiculous_version_name[] = "Mitigation";
 /* The ``DEFAULT'' port used for irc server connections. */
 int		irc_port = IRC_PORT;
 
-/* Set if ircII should usurp flow control, unset if not.  Probably bogus. */
-int		use_flow_control = 1;
-
-/* Set if ircII should turn on IEXTEN, unset to suppress */
-int		use_iexten = -1;
-
 /* 
  * When a numeric is being processed, this holds the negative value
  * of the numeric.  Its negative so if we pass it to do_hook, it can
@@ -192,8 +186,10 @@ char		*startup_file = NULL,		/* Set when epicrc loaded */
 		*default_channel = NULL,	/* Channel to join on connect */
 		nickname[NICKNAME_LEN + 1],	/* users nickname */
 		hostname[NAME_LEN + 1],		/* name of current host */
+#if 0
 		username[NAME_LEN + 1],		/* usernameof user */
 		userhost[NAME_LEN + 1],		/* userhost of user */
+#endif
 		*send_umode = NULL,		/* sent umode */
 		*last_notify_nick = (char *) 0;	/* last detected nickname */
 const char	empty_string[] = "",		/* just an empty string */
@@ -218,10 +214,6 @@ static		char	switch_help[] =
 #endif
 "      -B\tLoads your .ircrc file before you connect to a server.      \n\
       -d\tThe program should run in ``dumb mode'' (no fancy screen)   \n\
-      -f\tThe program won't mess with your flow control                \n\
-      -F\tThe program will mess with your flow control                \n\
-      -o\tThe program wll turn on IEXTEN terminal setting (^V/^O)     \n\
-      -O\tThe program will turn off IEXTEN (so you can bind ^V/^O)    \n\
       -h\tPrint this help message                                     \n\
       -q\tThe program will not load your .ircrc file                  \n\
       -s\tThe program will not connect to a server upon startup       \n\
@@ -292,8 +284,8 @@ void	irc_exit (int really_quit, const char *format, ...)
 #ifdef HAVE_PERL
 	perlstartstop(0);  /* In case there's perl code in the exit hook. */
 #endif
-#ifdef RUBY
-	rubystartstop(0);
+#ifdef HAVE_RUBY
+	ruby_startstop(0);
 #endif
 
 	close_all_servers(quit_message);
@@ -314,14 +306,15 @@ void	irc_exit (int really_quit, const char *format, ...)
 	/* Try to free as much memory as possible */
 	window_display = 0;
 /*	dumpcmd(NULL, NULL, NULL); */
-	flush_all_symbols();
-	flush_on_hooks();
-	set_lastlog_size(&value);
 	remove_channel(NULL, 0);
-	destroy_call_stack();
-	remove_bindings();
+	set_lastlog_size(&value);
 	delete_all_windows();
 	destroy_server_list();
+
+	destroy_call_stack();
+	remove_bindings();
+	flush_on_hooks();
+	flush_all_symbols();
 	window_display = old_window_display;
 	fprintf(stdout, "\r");
 	fflush(stdout);
@@ -363,7 +356,7 @@ static SIGNAL_HANDLER(coredump)
 									\n\
 									\n\
 * * * * * * * * * * * * * * * * * * * * * * * *				\n\
-IRC-II has trapped a critical protection error.				\n\
+EPIC has trapped a critical protection error.				\n\
 This is probably due to a bug in the program.				\n\
 									\n\
 If you have access to the 'BUG_FORM' in the ircII source distribution,	\n\
@@ -375,8 +368,8 @@ to it, please dont worry about filling it out.  You might try talking	\n\
 to the person who is in charge of IRC at your site and see if you can	\n\
 get them to help you.							\n\
 									\n\
-This version of IRC II is  --->[%s (%lu)]				\n\
-The date of release is     --->[%s]					\n\
+This version of EPIC is --->[%s (%lu)]					\n\
+The date of release is  --->[%s]					\n\
 									\n\
 * * * * * * * * * * * * * * * * * * * * * * * *				\n\
 The program will now terminate.						\n", irc_version, commit_id, internal_version);
@@ -493,7 +486,9 @@ static	void	parse_args (int argc, char **argv)
 	extern int optind;
 
 	*nickname = 0;
+#if 0
 	*username = 0;
+#endif
 
 	/* 
 	 * Its probably better to parse the environment variables
@@ -506,11 +501,11 @@ static	void	parse_args (int argc, char **argv)
 		{
 			if ((ptr = strchr(entry->pw_gecos, ',')))
 				*ptr = 0;
-			set_var_value(REALNAME_VAR, entry->pw_gecos, 0);
+			set_var_value(DEFAULT_REALNAME_VAR, entry->pw_gecos, 0);
 		}
 
 		if (entry->pw_name && *(entry->pw_name))
-			strlcpy(username, entry->pw_name, sizeof username);
+			set_var_value(DEFAULT_USERNAME_VAR, entry->pw_name, 0);
 
 		if (entry->pw_dir && *(entry->pw_dir))
 			malloc_strcpy(&my_path, entry->pw_dir);
@@ -530,46 +525,48 @@ static	void	parse_args (int argc, char **argv)
 	 * username.  I think everyone would have to agree this is the most
 	 * reasonable way to handle this.
 	 */
-	if (!*username)
+	if (empty(get_string_var(DEFAULT_USERNAME_VAR)))
 		if ((ptr = getenv("LOGNAME")) && *ptr)
-			strlcpy(username, ptr, sizeof username);
+			set_var_value(DEFAULT_USERNAME_VAR, ptr, 0);
 
 #ifndef ALLOW_USER_SPECIFIED_LOGIN
-	if (!*username)
+	if (empty(get_string_var(DEFAULT_USERNAME_VAR)))
 #endif
 		if ((ptr = getenv("IRCUSER")) && *ptr) 
-			strlcpy(username, ptr, sizeof username);
+			set_var_value(DEFAULT_USERNAME_VAR, ptr, 0);
 #ifdef ALLOW_USER_SPECIFIED_LOGIN
-		else if (*username)
+		else if (empty(get_string_var(DEFAULT_USERNAME_VAR)))
 			;
 #endif
 		else if ((ptr = getenv("USER")) && *ptr) 
-			strlcpy(username, ptr, sizeof username);
+			set_var_value(DEFAULT_USERNAME_VAR, ptr, 0);
 		else if ((ptr = getenv("HOME")) && *ptr)
 		{
 			char *ptr2 = strrchr(ptr, '/');
 			if (ptr2)
-				strlcpy(username, ptr2, sizeof username);
+				set_var_value(DEFAULT_USERNAME_VAR, ptr2, 0);
 			else
-				strlcpy(username, ptr, sizeof username);
+				set_var_value(DEFAULT_USERNAME_VAR, ptr, 0);
 		}
 		else
 		{
 			fprintf(stderr, "I dont know what your user name is.\n");
 			fprintf(stderr, "Set your LOGNAME environment variable\n");
-			fprintf(stderr, "and restart IRC II.\n");
+			fprintf(stderr, "and restart EPIC.\n");
 			exit(1);
 		}
 
 	if ((ptr = getenv("IRCNAME")))
-		set_var_value(REALNAME_VAR, ptr, 0);
+		set_var_value(DEFAULT_REALNAME_VAR, ptr, 0);
 	else if ((ptr = getenv("NAME")))
-		set_var_value(REALNAME_VAR, ptr, 0);
+		set_var_value(DEFAULT_REALNAME_VAR, ptr, 0);
+	else if ((ptr = getenv("REALNAME")))
+		set_var_value(DEFAULT_REALNAME_VAR, ptr, 0);
 	else
 	{
-		ptr = get_string_var(REALNAME_VAR);
+		ptr = get_string_var(DEFAULT_REALNAME_VAR);
 		if (!ptr || !*ptr)
-			set_var_value(REALNAME_VAR, "*Unknown*", 0);
+			set_var_value(DEFAULT_REALNAME_VAR, "*Unknown*", 0);
 	}
 
 	if ((ptr = getenv("HOME")))
@@ -671,7 +668,7 @@ static	void	parse_args (int argc, char **argv)
 	/*
 	 * Parse the command line arguments.
 	 */
-	while ((ch = getopt(argc, argv, "aBbc:dfFhH:l:L:n:oOp:qsSvxz:")) != EOF)
+	while ((ch = getopt(argc, argv, "aBbc:dhH:l:L:n:p:qsSvxz:")) != EOF)
 	{
 		switch (ch)
 		{
@@ -683,22 +680,6 @@ static	void	parse_args (int argc, char **argv)
 				irc_port = my_atol(optarg);
 				break;
 
-			case 'f': /* Use flow control */
-				use_flow_control = 1;
-				break;
-
-			case 'F': /* dont use flow control */
-				use_flow_control = 0;
-				break;
-
-			case 'o': /* Use IEXTEN */
-				use_iexten = 1;
-				break;
-
-			case 'O': /* dont use IEXTEN */
-				use_iexten = 0;
-				break;
-
 			case 'd': /* use dumb mode */
 				dumb_mode = 1;
 				break;
@@ -744,7 +725,7 @@ static	void	parse_args (int argc, char **argv)
 
 			case 'z':
 #ifdef ALLOW_USER_SPECIFIED_LOGIN
-				strlcpy(username, optarg, sizeof username);
+				set_var_value(DEFAULT_USERNAME_VAR, optarg, 0);
 #endif
 				break;
 
@@ -778,7 +759,8 @@ static	void	parse_args (int argc, char **argv)
 	 * so do a final check on whatever nickname we're going to use.
 	 */
 	if (!*nickname)
-		strlcpy(nickname, username, sizeof nickname);
+		strlcpy(nickname, get_string_var(DEFAULT_USERNAME_VAR), 
+				sizeof nickname);
 
 	for (; *argv; argc--, argv++)
 		if (**argv)
@@ -798,7 +780,7 @@ static	void	parse_args (int argc, char **argv)
 	if (!check_nickname(nickname, 1))
 	{
 		fprintf(stderr, "Invalid nickname: [%s]\n", nickname);
-		fprintf(stderr, "Please restart IRC II with a valid nickname\n");
+		fprintf(stderr, "Please restart EPIC with a valid nickname\n");
 		exit(1);
 	}
 
@@ -903,7 +885,7 @@ static	int		level = 0,
 			level, what, caller[level-1], caller[level-2], 
 					caller[level-3], caller[level-4]);
 	    if (level % 50 == 0)
-		panic("Ahoy there matey!  Abandon ship!");
+		panic(1, "Ahoy there matey!  Abandon ship!");
 	}
 	else if (level && (last_warn - level == 5))
 	    last_warn -= 5;
@@ -1165,7 +1147,12 @@ int 	main (int argc, char *argv[])
 	check_valid_user();
 	check_invalid_host();
 
+#ifdef WITH_THREADED_STDOUT
+	tio_init();
+#endif
+
 	init_levels();
+	init_transforms();
 	init_variables_stage1();
 	parse_args(argc, argv);
 	init_binds();
@@ -1180,16 +1167,35 @@ int 	main (int argc, char *argv[])
 	fprintf(stderr, "Version (%s), Commit Id (%lu) -- Date (%s)\n", 
 				irc_version, commit_id, internal_version);
 	fprintf(stderr, "%s\n", compile_info);
-	fprintf(stderr, "Process [%d]", getpid());
-	if (isatty(0))
-		fprintf(stderr, " connected to tty [%s]", ttyname(0));
-	else
-		dumb_mode = 1;
-	fprintf(stderr, "\n");
 
 	/* If we're a bot, do the bot thing. */
-	if (!use_input && fork())
-		_exit(0);
+	if (!use_input)
+	{
+		pid_t	child_pid;
+
+		child_pid = fork();
+		if (child_pid == -1)
+		{
+			fprintf(stderr, "Could not fork a child process: %s\n",
+					strerror(errno));
+			_exit(1);
+		}
+		else if (child_pid > 0)
+		{
+			fprintf(stderr, "Process [%d] running in background\n",
+					child_pid);
+			_exit(0);
+		}
+	}
+	else
+	{
+		fprintf(stderr, "Process [%d]", getpid());
+		if (isatty(0))
+			fprintf(stderr, " connected to tty [%s]", ttyname(0));
+		else
+			dumb_mode = 1;
+		fprintf(stderr, "\n");
+	}
 
 	/* these should be taken by both dumb and smart displays */
 	my_signal(SIGSEGV, coredump);
@@ -1213,7 +1219,7 @@ int 	main (int argc, char *argv[])
 		init_variables_stage2();
 		build_status(NULL);
 		permit_status_update(1);
-		update_input(UPDATE_ALL);
+		update_input(NULL, UPDATE_ALL);
 	}
 	else
 	{
