@@ -1,4 +1,4 @@
-/* $EPIC: keys.c,v 1.52 2006/10/14 16:57:09 jnelson Exp $ */
+/* $EPIC: keys.c,v 1.55 2007/07/20 22:29:32 jnelson Exp $ */
 /*
  * keys.c:  Keeps track of what happens whe you press a key.
  *
@@ -49,11 +49,12 @@
 #include "window.h"
 #include "timer.h"
 #include "reg.h"
+#include "server.h"
 
 /* This is a typedef for a function used with the /BIND system.  The
  * functions all live in input.c.  Following it is a macro to quickly define
  * functions to handle keybindings. */
-typedef void (*BindFunction) (unsigned char, unsigned char *);
+typedef void (*BindFunction) (unsigned int, unsigned char *);
 
 /* This is the structure used to hold binding functions. */
 struct Binding {
@@ -208,7 +209,7 @@ void init_binds (void) {
     ADDBIND("QUIT_IRC",			    irc_quit			    );
     ADDBIND("QUOTE_CHARACTER",		    quote_char			    );
     ADDBIND("REFRESH_INPUTLINE",	    refresh_inputline		    );
-    ADDBIND("REFRESH_SCREEN",		    (BindFunction) refresh_screen   );
+    ADDBIND("REFRESH_SCREEN",		    refresh_screen   		    );
     ADDBIND("REFRESH_STATUS",		    (BindFunction) update_all_status);
     ADDBIND("RESET_LINE",		    input_reset_line		    );
     ADDBIND("REVERSE",			    insert_reverse		    );
@@ -245,7 +246,7 @@ struct Key	*construct_keymap	(struct Key *);
 int		clean_keymap		(struct Key *);
 unsigned char	*bind_string_compress	(const unsigned char *, int *);
 unsigned char	*bind_string_decompress	(unsigned char *, const unsigned char *, int);
-static int	bind_string		(const u_char *, const char *, char *);
+static int	bind_string		(const unsigned char *, const char *, char *);
 struct Key	*find_sequence		(const unsigned char *, int);
 void		show_all_bindings	(struct Key *, const unsigned char *, size_t);
 void		show_all_rbindings	(struct Key *, const unsigned char *, int, struct Binding *);
@@ -421,6 +422,40 @@ void *	timeout_keypress (void *lastp, Timeval pressed) {
     return (void *)last; /* still waiting.. */
 }
 
+/* do_input_timeouts: walk through the list of screens and potentially do
+ * timeouts for input */
+ /* XXX This doesn't belong here -- maybe in keys.c? */
+int     do_input_timeouts (void *ignored)
+{
+    Screen *oldscreen = last_input_screen;
+    Screen *screen;
+    int server;
+
+    if (!foreground)
+        return 0;               /* Not a lot of input.. :) */
+
+    /* I hope this is sufficient to allow input to be processed.  I took
+     * a lot of the guts from do_screens().  Here's hoping I didn't
+     * break anything. ;) */
+    for (screen = screen_list; screen; screen = screen->next) {
+        if (screen->alive) {
+            server = from_server;
+            last_input_screen = screen;
+            output_screen = screen;
+            make_window_current(screen->current_window);
+            from_server = current_window->server;
+            screen->last_key = timeout_keypress(screen->last_key,
+					        screen->last_press);
+            from_server = server;
+        }
+    }
+    output_screen = last_input_screen = oldscreen;
+    return 0;
+}
+
+
+/**************************************************************************/
+
 struct Key *construct_keymap (struct Key *owner) {
     int c;
     struct Key *map = new_malloc(sizeof(struct Key) * KEYMAP_SIZE);
