@@ -1,4 +1,4 @@
-/* $EPIC: lastlog.c,v 1.64 2006/10/13 21:58:02 jnelson Exp $ */
+/* $EPIC: lastlog.c,v 1.69 2007/08/02 16:36:58 jnelson Exp $ */
 /*
  * lastlog.c: handles the lastlog features of irc. 
  *
@@ -44,6 +44,7 @@
 #include "numbers.h"
 #include "functions.h"
 #include "reg.h"
+#include "alias.h"
 
 typedef struct	lastlog_stru
 {
@@ -61,6 +62,7 @@ typedef struct	lastlog_stru
 static	intmax_t global_lastlog_refnum = 0;
 
 static int	show_lastlog (Lastlog **l, int *skip, int *number, Mask *level_mask, char *match, regex_t *reg, int *max, const char *target, int mangler);
+static void	remove_lastlog_item (Window *window, Lastlog *being_removed);
 
 Lastlog *	lastlog_oldest = NULL;
 Lastlog *	lastlog_newest = NULL;
@@ -76,19 +78,6 @@ static	Mask	notify_mask;
 	Mask *	old_server_lastlog_mask = NULL;
 	Mask 	current_window_mask;
 
-/*
- * warn_lastlog_level: tells the user what levels are available.
- */
-static void	warn_lastlog_levels (void)
-{
-	char *buffer;
-
-	strlcpy(buffer, "Valid levels: ", sizeof buffer);
-	buffer = get_all_levels();
-	say("%s", buffer);
-	new_free(&buffer);	
-}
-
 /*
  * set_lastlog_mask: called whenever a "SET LASTLOG_LEVEL" is done.  It
  * parses the settings and sets the lastlog_mask variable appropriately.  It
@@ -233,49 +222,72 @@ intmax_t	add_to_lastlog (Window *window, const char *line)
 		new_l->visible = 0;
 
 	/* * * */
-#if 0
-	lastlog_newest->newer = new_l;
-	new_l->older = lastlog_newest;
-	lastlog_newest = new_l;
-
-	if (!lastlog_oldest)
-		lastlog_oldest = lastlog_newest;
-#endif
-
-	/* * * */
 	return new_l->refnum;
 }
 
 
 void 	trim_lastlog (Window *window)
 {
-	Lastlog *new_oldest;
-	Lastlog *being_removed;
-
 	while (window->lastlog_oldest)
 	{
 		/* All done! */
 		if (window->lastlog_size <= window->lastlog_max)
 			return;
 
-		being_removed = window->lastlog_oldest;
-		new_oldest = being_removed->newer;
-		window->lastlog_oldest = new_oldest;
-		if (new_oldest)
-			new_oldest->older = NULL;
-		else
-			window->lastlog_newest = NULL;
-		if (being_removed->visible)
-			window->lastlog_size--;
-		new_free((char **)&being_removed->msg);
-		new_free((char **)&being_removed->target);
-		new_free((char **)&being_removed);
+		remove_lastlog_item(window, window->lastlog_oldest);
 	}
 
 	/* Uh, the lastlog must be empty... */
 	window->lastlog_size = 0;
 }
 
+void	omit_from_lastlog (Window *window, const char *string)
+{
+	Lastlog *item;
+
+	item = window->lastlog_oldest;
+	while (item)
+	{
+		if (stristr(item->msg, string))
+		{
+			remove_lastlog_item(window, item);
+			item = window->lastlog_oldest;	/* Start over */
+			break;
+		}
+	}
+}
+
+static void	remove_lastlog_item (Window *window, Lastlog *being_removed)
+{
+	if (being_removed == window->lastlog_oldest)
+	{
+		window->lastlog_oldest = being_removed->newer;
+		if (window->lastlog_oldest)
+			window->lastlog_oldest->older = NULL;
+	}
+	else if (being_removed->older)
+		being_removed->older->newer = being_removed->newer;
+	else
+		panic(1, "Lastlog item [%s] being removed is not oldest and has no older.", being_removed->msg);
+
+	if (being_removed == window->lastlog_newest)
+	{
+		window->lastlog_newest = being_removed->older;
+		if (window->lastlog_newest)
+			window->lastlog_newest->newer = NULL;
+	}
+	else if (being_removed->newer)
+		being_removed->newer->older = being_removed->older;
+	else
+		panic(1, "Lastlog item [%s] being removed is not newest and has no newest.", being_removed->msg);
+
+	if (being_removed->visible)
+		window->lastlog_size--;
+	new_free((char **)&being_removed->msg);
+	new_free((char **)&being_removed->target);
+	new_free((char **)&being_removed);
+}
+
 /*
  * set_lastlog_size: sets up a lastlog buffer of size given.  If the lastlog
  * has gotten larger than it was before, all newer lastlog entries remain.
@@ -310,6 +322,7 @@ void	set_lastlog_size (void *stuff)
  *	-max <number>		Only show first <number> matches
  *	-skip <number>		Skip this many leading lastlog entries
  *	-number <number>	Only search this many lastlog entries
+ *	-rewrite "<expando>"	Rewrite each line with the expando.
  *	-<LEVEL>		Add <LEVEL> to "level mask"
  *	--<LEVEL>		Remove <LEVEL> from "level mask"
  *	-ALL			Add all levels to "level mask"
@@ -384,12 +397,14 @@ BUILT_IN_COMMAND(lastlog)
 	FILE *		outfp = NULL;
 	int		mangler = 0;
 	int		lc;
+	char *		rewrite = NULL;
 
 	lc = message_setall(0, NULL, LEVEL_OTHER);
 	cnt = current_window->lastlog_size;
 	save_mask = current_window->lastlog_mask;
 	mask_unsetall(&current_window->lastlog_mask);
 	mask_unsetall(&level_mask);
+	rewrite = get_string_var(LASTLOG_REWRITE_VAR);
 
 	while ((arg = new_next_arg(args, &args)) != NULL)
 	{
@@ -525,6 +540,10 @@ BUILT_IN_COMMAND(lastlog)
 			goto bail;
 		}
 	    }
+	    else if (!my_strnicmp(arg, "-REWRITE", len))
+	    {
+		rewrite = new_next_arg(args, &args);
+	    }
 	    else if (!my_strnicmp(arg, "-", 1))
 	    {
 		int	i = str_to_level(arg+1);
@@ -703,7 +722,26 @@ BUILT_IN_COMMAND(lastlog)
 
 		if (counter)
 		{
-			file_put_it(outfp, "%s", l->msg);
+			if (rewrite)
+			{
+				unsigned char *n, vitals[10240];
+
+				snprintf(vitals, sizeof(vitals),
+					"%ld %ld %ld %ld . . . %s %s",
+						(long)l->refnum,
+						(long)l->when,
+						(long)l->winref,
+						(long)l->level,
+						l->target?l->target:".",
+						l->msg?l->msg:".");
+
+				n = expand_alias(rewrite, vitals);
+				file_put_it(outfp, "%s", n);
+				new_free(&n);
+			}
+			else
+				file_put_it(outfp, "%s", l->msg);
+
 			lastshown = l;
 			counter--;
 			if (counter == 0 && before != -1 && separator)
@@ -768,7 +806,26 @@ BUILT_IN_COMMAND(lastlog)
 		}
 		if (counter)
 		{
-			file_put_it(outfp, "%s", l->msg);
+			if (rewrite)
+			{
+				unsigned char *n, vitals[10240];
+
+				snprintf(vitals, sizeof(vitals),
+					"%ld %ld %ld %ld . . . %s %s",
+						(long)l->refnum,
+						(long)l->when,
+						(long)l->winref,
+						(long)l->level,
+						l->target?l->target:".",
+						l->msg?l->msg:".");
+
+				n = expand_alias(rewrite, vitals);
+				file_put_it(outfp, "%s", n);
+				new_free(&n);
+			}
+			else
+				file_put_it(outfp, "%s", l->msg);
+
 			lastshown = l;
 			counter--;
 			if (counter == 0 && before != -1 && separator)
@@ -1079,7 +1136,7 @@ int	remove_lastlog_item (Lastlog *item)
 	if (item == lastlog_oldest)
 	{
 		if (item->older != NULL)
-			panic("Oldest lastlog item %jd has older item %jd",
+			panic(1, "Oldest lastlog item %jd has older item %jd",
 				item->refnum, item->older->refnum);
 
 		item->newer->older = NULL;
@@ -1089,7 +1146,7 @@ int	remove_lastlog_item (Lastlog *item)
 	else if (item == lastlog_newest)
 	{
 		if (item->newer != NULL)
-			panic("Newest lastlog item %jd has newer item %jd",
+			panic(1, "Newest lastlog item %jd has newer item %jd",
 				item->refnum, item->newer->refnum);
 
 		item->older->newer = NULL;
