@@ -1,11 +1,11 @@
-/* $EPIC: newio.c,v 1.61 2006/10/08 14:00:08 jnelson Exp $ */
+/* $EPIC: newio.c,v 1.67 2007/07/20 22:29:32 jnelson Exp $ */
 /*
  * newio.c:  Passive, callback-driven IO handling for sockets-n-stuff.
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1997, 2005 EPIC Software Labs.
+ * Copyright  1997, 2007 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -73,6 +73,7 @@ typedef	struct	myio_struct
 	void	(*callback) (int vfd);
 	int	(*io_callback) (int vfd, int quiet);
 	int	quiet;
+	int	server;			/* For message routing */
 }           MyIO;
 
 static	MyIO **	io_rec = NULL;
@@ -135,7 +136,7 @@ static	int	get_new_vfd (int channel)
 static int	get_channel_by_vfd (int vfd)
 {
 	if (!io_rec[vfd])
-		panic("get_channel_by_vfd(%d): vfd is not set up!");
+		panic(1, "get_channel_by_vfd(%d): vfd is not set up!");
 	return io_rec[vfd]->channel;
 }
 #define CHANNEL(vfd) get_channel_by_vfd(vfd);
@@ -145,6 +146,14 @@ static int	get_channel_by_vfd (int vfd)
 #define get_new_vfd(channel) channel
 #endif
 
+int	get_server_by_vfd (int vfd)
+{
+	if (!io_rec[vfd])
+		panic(1, "get_server_by_vfd(%d): vfd is not set up!");
+	return io_rec[vfd]->server;
+}
+/* #define SRV(vfd) get_server_by_vfd(vfd) */ /* Defined in newio.h */
+#define CSRV(channel) get_server_by_vfd(VFD(channel))
 
 /**************************************************************************/
 /*
@@ -164,7 +173,7 @@ int	dgets_buffer (int channel, void *data, ssize_t len)
 	klock();
 	vfd = VFD(channel);
 	if (!(ioe = io_rec[vfd]))
-		panic("dgets called on unsetup channel %d", channel);
+		panic(1, "dgets called on unsetup channel %d", channel);
 
 	/* 
 	 * An old exploit just sends us characters every .8 seconds without
@@ -173,7 +182,8 @@ int	dgets_buffer (int channel, void *data, ssize_t len)
 	if (ioe->segments > MAX_SEGMENTS)
 	{
 		if (!ioe->quiet)
-		    syserr("dgets_buffer: Too many read()s on channel [%d] "
+		    syserr(ioe->server, 
+			"dgets_buffer: Too many read()s on channel [%d] "
 			"without a newline -- shutting off bad peer", channel);
 		ioe->error = -1;
 		ioe->clean = 0;
@@ -266,18 +276,19 @@ ssize_t	dgets (int vfd, char *buf, size_t buflen, int buffer)
 
 	if (buflen == 0)
 	{
-	    syserr("dgets: Destination buffer for vfd [%d] is zero length."
+	    syserr(SRV(vfd),
+			"dgets: Destination buffer for vfd [%d] is zero length."
 			" This is surely a bug.", vfd);
 	    return -1;
 	}
 
 	if (!(ioe = io_rec[vfd]))
-		panic("dgets called on unsetup vfd %d", vfd);
+		panic(1, "dgets called on unsetup vfd %d", vfd);
 
 	if (ioe->error)
 	{
 	    if (!ioe->quiet)
-	       syserr("dgets: Reporting exception for vfd [%d]", vfd);
+	       syserr(SRV(vfd), "dgets: fd [%d] must be closed", vfd);
 	    return -1;
 	}
 
@@ -413,7 +424,7 @@ static	int	polls = 0;
 		if (polls++ > 10000)
 		{
 		    dump_timers();
-		    panic("Stuck in a polling loop. Help!");
+		    panic(1, "Stuck in a polling loop. Help!");
 		}
 		return 0;		/* Timers are more important */
 	    }
@@ -464,7 +475,7 @@ void	init_newio (void)
 	int	max_fd = IO_ARRAYLEN;
 
 	if (io_rec)
-		panic("init_newio() called twice.");
+		panic(1, "init_newio() called twice.");
 
 	io_rec = (MyIO **)new_malloc(sizeof(MyIO *) * max_fd);
 	for (vfd = 0; vfd < max_fd; vfd++)
@@ -505,7 +516,7 @@ int	my_iswritable (int vfd, double seconds)
  * Set up its input buffer
  * Returns an vfd!
  */
-int 	new_open (int channel, void (*callback) (int), int io_type, int quiet)
+int 	new_open (int channel, void (*callback) (int), int io_type, int quiet, int server)
 {
 	MyIO *ioe;
 	int	vfd;
@@ -537,6 +548,7 @@ int 	new_open (int channel, void (*callback) (int), int io_type, int quiet)
 	ioe->clean = 1;
 	ioe->held = 0;
 	ioe->quiet = quiet;
+	ioe->server = server;
 
 	if (io_type == NEWIO_READ)
 		ioe->io_callback = unix_read;
@@ -555,7 +567,7 @@ int 	new_open (int channel, void (*callback) (int), int io_type, int quiet)
 		ioe->io_callback = ssl_connect;
 #endif
 	else
-		panic("New_open doesn't recognize io type %d", io_type);
+		panic(1, "New_open doesn't recognize io type %d", io_type);
 
 	ioe->callback = callback;
 
@@ -666,7 +678,7 @@ static int	unix_close (int channel, int quiet)
 	if (close(channel))
 	{
 		if (!quiet)
-		   syserr("unix_close: close(%d) failed: %s", 
+		   syserr(CSRV(channel), "unix_close: close(%d) failed: %s", 
 			channel, strerror(errno));
 		return -1;
 	}
@@ -682,13 +694,13 @@ static int	unix_read (int channel, int quiet)
 	if (c == 0)
 	{
 		if (!quiet)
-		   syserr("unix_read: EOF for fd %d ", channel);
+		   syserr(CSRV(channel), "unix_read: EOF for fd %d ", channel);
 		return 0;
 	}
 	else if (c < 0)
 	{
 		if (!quiet)
-		   syserr("unix_read: read(%d) failed: %s", 
+		   syserr(CSRV(channel), "unix_read: read(%d) failed: %s", 
 				channel, strerror(errno));
 		return -1;
 	}
@@ -696,7 +708,7 @@ static int	unix_read (int channel, int quiet)
 	if (dgets_buffer(channel, buffer, c))
 	{
 		if (!quiet)
-		   syserr("unix_read: dgets_buffer(%d, %*s) failed",
+		   syserr(CSRV(channel), "unix_read: dgets_buffer(%d, %*s) failed",
 				channel, c, buffer);
 		return -1;
 	}
@@ -713,13 +725,13 @@ static int	unix_recv (int channel, int quiet)
 	if (c == 0)
 	{
 		if (!quiet)
-		   syserr("unix_recv: EOF for fd %d ", channel);
+		   syserr(CSRV(channel), "unix_recv: EOF for fd %d ", channel);
 		return 0;
 	}
 	else if (c < 0)
 	{
 		if (!quiet)
-		   syserr("unix_recv: read(%d) failed: %s", 
+		   syserr(CSRV(channel), "unix_recv: read(%d) failed: %s", 
 				channel, strerror(errno));
 		return -1;
 	}
@@ -727,7 +739,7 @@ static int	unix_recv (int channel, int quiet)
 	if (dgets_buffer(channel, buffer, c))
 	{
 		if (!quiet)
-		   syserr("unix_recv: dgets_buffer(%d, %*s) failed",
+		   syserr(CSRV(channel), "unix_recv: dgets_buffer(%d, %*s) failed",
 				channel, c, buffer);
 		return -1;
 	}
@@ -749,17 +761,20 @@ static int	unix_accept (int channel, int quiet)
 		FD_SET(channel, &fdset);
 		if (select(channel + 1, &fdset, NULL, NULL, NULL) < 0)
 		{
-			if (!quiet)
-			   syserr("unix_accept: select(%d) failed: %s",
-				channel, strerror(errno));
+		    if (!quiet)
+			syserr(CSRV(channel), "unix_accept: select(%d) failed"
+				": %s", channel, strerror(errno));
 		}
 	}
 #endif
 
 	len = sizeof(addr);
 	if ((newfd = Accept(channel, (SA *)&addr, &len)) < 0)
-		if (!quiet)
-		   syserr("unix_accept: Accept(%d) failed", channel);
+	{
+	    if (!quiet)
+		syserr(CSRV(channel), "unix_accept: Accept(%d) failed", 
+					": %s", channel, strerror(errno));
+	}
 
 	dgets_buffer(channel, &newfd, sizeof(newfd));
 	dgets_buffer(channel, &addr, sizeof(addr));
@@ -768,6 +783,8 @@ static int	unix_accept (int channel, int quiet)
 
 static int	unix_connect (int channel, int quiet)
 {
+	int	sockerr;
+	int	gso_result;
 	int	gsn_result;
 	SS	localaddr;
 	int	gpn_result;
@@ -783,12 +800,23 @@ static int	unix_connect (int channel, int quiet)
 		if (select(channel + 1, NULL, &fdset, NULL, NULL) < 0)
 		{
 			if (!quiet)
-			   syserr("unix_connect: select(%d) failed: %s",
+			   syserr(CSRV(channel), 
+				"unix_connect: select(%d) failed: %s",
 				channel, strerror(errno));
 		}
 	}
 #endif
 
+	/* * */
+	len = sizeof(sockerr);
+	errno = 0;
+	getsockopt(channel, SOL_SOCKET, SO_ERROR, &sockerr, &len);
+	gso_result = errno;
+
+	dgets_buffer(channel, &sockerr, sizeof(sockerr));
+	dgets_buffer(channel, &gso_result, sizeof(gso_result));
+
+	/* * */
 	len = sizeof(localaddr);
 	errno = 0;
 	getsockname(channel, (SA *)&localaddr, &len);
@@ -797,6 +825,7 @@ static int	unix_connect (int channel, int quiet)
 	dgets_buffer(channel, &gsn_result, sizeof(gsn_result));
 	dgets_buffer(channel, &localaddr, sizeof(localaddr));
 
+	/* * */
 	len = sizeof(remoteaddr);
 	errno = 0;
 	getpeername(channel, (SA *)&remoteaddr, &len);
@@ -820,19 +849,18 @@ static void	new_io_event (int vfd)
 	int	c;
 
 	if (!(ioe = io_rec[vfd]))
-		panic("new_io_event: vfd [%d] isn't set up!", vfd);
+		panic(1, "new_io_event: vfd [%d] isn't set up!", vfd);
 
 	/* If it's dirty, something is very wrong. */
 	if (!ioe->clean)
-		panic("new_io_event: vfd [%d] hasn't been cleaned yet", vfd);
+		panic(1, "new_io_event: vfd [%d] hasn't been cleaned yet", vfd);
 
 	if ((c = ioe->io_callback(vfd, ioe->quiet)) <= 0)
 	{
 		ioe->error = -1;
 		ioe->clean = 0;
 		if (!ioe->quiet)
-		   syserr("new_io_event: io_callback(%d) "
-				"said fd should be closed", vfd);
+		   syserr(SRV(vfd), "new_io_event: fd %d must be closed", vfd);
 
 		if (x_debug & DEBUG_INBOUND) 
 			yell("VFD [%d] FAILED [%d]", vfd, c);
@@ -897,8 +925,9 @@ static	int	kdoit (Timeval *timeout)
 		    retval = select(channel + 1, &working_rd, NULL, NULL, &t);
 		    if (retval < 0)
 		    {
-			syserr("kdoit(select): Closing channel %d because: %s", 
-						channel, strerror(errno));
+			syserr(-1, "kdoit(select): "
+					"Closing channel %d because: %s", 
+					channel, strerror(errno));
 			FD_CLR(channel, &readables);
 			FD_CLR(channel, &writables);
 			new_close(channel);
@@ -907,11 +936,12 @@ static	int	kdoit (Timeval *timeout)
 		}
 
 		if (!foundit)
-			panic("kdoit(select): Select says I have a bad file "
+			panic(1, "kdoit(select): Select says I have a bad file "
 				"descriptor but I can't find it!");
 	    }
 	    else
-		syserr("kdoit(select): select() failed: %s", strerror(errno));
+		syserr(-1, "kdoit(select): select() failed: %s", 
+				strerror(errno));
 	}
 	else if (retval > 0)
 	{
@@ -985,7 +1015,8 @@ static void kinit (void)
 { 
         if ((kqueue_fd = kqueue()) < 0)
 	{
-		syserr("kinit(kqueue): kqueue() failed: %s", strerror(errno));
+		syserr(-1, "kinit(kqueue): kqueue() failed: %s", 
+				strerror(errno));
 		irc_exit(1, "Your system doesn't support kqueue(2)");
 	}
 }
@@ -996,7 +1027,7 @@ static  void    kread (int vfd)
 	EV_SET(&ev, CHANNEL(vfd), EVFILT_READ, EV_ADD|EV_ENABLE, 0, 0, NULL);
 	if (kevent(kqueue_fd, &ev, 1, NULL, 0, &kqueue_poll))
 	{
-		syserr("kread(kqueue): kevent(%d) failed: %s", 
+		syserr(SRV(vfd), "kread(kqueue): kevent(%d) failed: %s", 
 				CHANNEL(vfd), strerror(errno));
 	}
 }
@@ -1008,7 +1039,7 @@ static  void    knoread (int vfd)
 	if (kevent(kqueue_fd, &ev, 1, NULL, 0, &kqueue_poll))
 	{
 	    if (errno != ENOENT)
-		syserr("knoread(kqueue): kevent(%d) failed: %s", 
+		syserr(SRV(vfd), "knoread(kqueue): kevent(%d) failed: %s", 
 				CHANNEL(vfd), strerror(errno));
 	}
 }
@@ -1020,7 +1051,7 @@ static  void    kholdread (int vfd)
 	if (kevent(kqueue_fd, &ev, 1, NULL, 0, &kqueue_poll))
 	{
 	    if (errno != ENOENT)
-		syserr("kholdread(kqueue): kevent(%d) failed: %s", 
+		syserr(SRV(vfd), "kholdread(kqueue): kevent(%d) failed: %s", 
 				CHANNEL(vfd), strerror(errno));
 	}
 }
@@ -1031,7 +1062,7 @@ static  void    kunholdread (int vfd)
 	EV_SET(&ev, CHANNEL(vfd), EVFILT_READ, EV_ENABLE, 0, 0, NULL);
 	if (kevent(kqueue_fd, &ev, 1, NULL, 0, &kqueue_poll))
 	{
-		syserr("kunholdread(kqueue): kevent(%d) failed: %s", 
+		syserr(SRV(vfd), "kunholdread(kqueue): kevent(%d) failed: %s", 
 				CHANNEL(vfd), strerror(errno));
 	}
 }
@@ -1042,7 +1073,7 @@ static  void    kwrite (int vfd)
 	EV_SET(&ev, CHANNEL(vfd), EVFILT_WRITE, EV_ADD|EV_ENABLE, 0, 0, NULL);
 	if (kevent(kqueue_fd, &ev, 1, NULL, 0, &kqueue_poll))
 	{
-		syserr("kwrite(kqueue): kevent(%d) failed: %s", 
+		syserr(SRV(vfd), "kwrite(kqueue): kevent(%d) failed: %s", 
 				CHANNEL(vfd), strerror(errno));
 	}
 }
@@ -1054,7 +1085,7 @@ static  void    knowrite (int vfd)
 	if (kevent(kqueue_fd, &ev, 1, NULL, 0, &kqueue_poll))
 	{
 	    if (errno != ENOENT)
-		syserr("knowrite(kqueue): kevent(%d) failed: %s", 
+		syserr(SRV(vfd), "knowrite(kqueue): kevent(%d) failed: %s", 
 				CHANNEL(vfd), strerror(errno));
 	}
 }
@@ -1079,7 +1110,7 @@ static	int	kdoit (Timeval *timeout)
 	}
 
 	if (retval < 0 && errno != EINTR)
-		syserr("kdoit(kqueue): kevent(NULL) failed: %s", 
+		syserr(-1, "kdoit(kqueue): kevent(NULL) failed: %s", 
 					strerror(errno));
 	else if (retval > 0)
 	{
@@ -1200,7 +1231,7 @@ static	int	kdoit (Timeval *timeout)
 	retval = poll(polls, global_max_vfd + 1, ms);
 
 	if (retval < 0 && errno != EINTR)
-		syserr("kdoit(poll): poll() failed: %s", strerror(errno));
+		syserr(-1, "kdoit(poll): poll() failed: %s", strerror(errno));
 	else if (retval > 0)
 	{
 		for (vfd = 0; vfd <= global_max_vfd; vfd++)
@@ -1284,15 +1315,15 @@ static	void *	pthread_io_event (void *vvfd)
 	new_free(&vvfd);
 
 	if ((err = pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL)))
-		syserr("pthread_io_event: pthread_setcancelstate(%d, PTHREAD_CANCEL_ENABLE) failed: %s", vfd, strerror(err));
+		syserr(SRV(vfd), "pthread_io_event: pthread_setcancelstate(%d, PTHREAD_CANCEL_ENABLE) failed: %s", vfd, strerror(err));
 
 	if ((err = pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL)))
-		syserr("pthread_io_event: pthread_setcanceltype(%d, PTHREAD_CANCEL_ASYNCHRONOUS) failed: %s", vfd, strerror(err));
+		syserr(SRV(vfd), "pthread_io_event: pthread_setcanceltype(%d, PTHREAD_CANCEL_ASYNCHRONOUS) failed: %s", vfd, strerror(err));
 
 	new_io_event(vfd);
 	klock();
 	if ((err = pthread_cond_signal(&cond)))
-		syserr("pthread_io_event: pthread_cond_signal(%d) failed: %s", 
+		syserr(SRV(vfd), "pthread_io_event: pthread_cond_signal(%d) failed: %s", 
 				vfd, strerror(err));
 	kunlock();
 	pthread_exit(NULL);
@@ -1314,18 +1345,18 @@ static void	kinit (void)
 	}
 
 	if ((err = pthread_mutex_init(&mutex, NULL)))
-		syserr("kinit(pthread): pthread_mutex_init() failed: %s",
+		syserr(-1, "kinit(pthread): pthread_mutex_init() failed: %s",
 				strerror(err));
 	klock();
 	if ((pthread_cond_init(&cond, NULL)))
-		syserr("kinit(pthread): pthread_cond_init() failed: %s",
+		syserr(-1, "kinit(pthread): pthread_cond_init() failed: %s",
 				strerror(err));
 }
  
 static  void    kread (int vfd)
 {
 	if (pfd[vfd].active)
-		panic("vfd [%d] already is active (reading)", vfd);
+		panic(1, "vfd [%d] already is active (reading)", vfd);
 }
  
 static  void    knoread (int vfd)
@@ -1336,11 +1367,11 @@ static  void    knoread (int vfd)
 		return;
 
 	if ((err = pthread_cancel(pfd[vfd].pthread)))
-		syserr("knoread(pthread): pthread_cancel(%d) failed: %s",
+		syserr(SRV(vfd), "knoread(pthread): pthread_cancel(%d) failed: %s",
 				vfd, strerror(err));
 
 	if ((err = pthread_join(pfd[vfd].pthread, NULL)))
-		syserr("knoread(pthread): pthread_join(%d) failed: %s",
+		syserr(SRV(vfd), "knoread(pthread): pthread_join(%d) failed: %s",
 				vfd, strerror(err));
 
 	pfd[vfd].active = 0;
@@ -1369,7 +1400,7 @@ static  void    kunholdread (int vfd)
 static  void    kwrite (int vfd)
 {
 	if (pfd[vfd].active)
-		panic("vfd [%d] already is active (writing)", vfd);
+		panic(1, "vfd [%d] already is active (writing)", vfd);
 }
 
 static  void    knowrite (int vfd)
@@ -1380,10 +1411,10 @@ static  void    knowrite (int vfd)
 		return;
 
 	if ((err = pthread_cancel(pfd[vfd].pthread)))
-		syserr("knowrite: pthread_cancel(%d) failed: %s",
+		syserr(SRV(vfd), "knowrite: pthread_cancel(%d) failed: %s",
 				vfd, strerror(err));
 	if ((err = pthread_join(pfd[vfd].pthread, NULL)))
-		syserr("knowrite: pthread_join(%d) failed: %s",
+		syserr(SRV(vfd), "knowrite: pthread_join(%d) failed: %s",
 				vfd, strerror(err));
 
 	pfd[vfd].active = 0;
@@ -1395,20 +1426,20 @@ static	void	kcleaned (int vfd)
 	int *	mvfd;
 
 	if (!pthread_equal(pthread_self(), global))
-		panic("kcleaned not called from global thread");
+		panic(1, "kcleaned not called from global thread");
 
 	if (pfd[vfd].active == 1)
-		panic("vfd [%d] is already active (kcleaned)", vfd);
+		panic(1, "vfd [%d] is already active (kcleaned)", vfd);
 
 	if (io_rec[vfd]->clean == 0)
-		panic("vfd [%d] is not really clean.", vfd);
+		panic(1, "vfd [%d] is not really clean.", vfd);
 
 	mvfd = new_malloc(sizeof *mvfd);
 	*mvfd = vfd;
 
 	if ((err = pthread_create(&pfd[vfd].pthread, NULL, 
 					pthread_io_event, mvfd)))
-		syserr("kcleaned: pthread_create(%d) failed: %s",
+		syserr(SRV(vfd), "kcleaned: pthread_create(%d) failed: %s",
 				vfd, strerror(err));
 
 	vfd[pfd].active = 1;
@@ -1425,7 +1456,7 @@ static	int	kdoit (Timeval *timeout)
 	int	err;
 
 	if (!pthread_equal(pthread_self(), global))
-		panic("kdoit not called from global thread");
+		panic(1, "kdoit not called from global thread");
 
 	clock_gettime(CLOCK_REALTIME, &right_now);
 	right_now.tv_nsec += (timeout->tv_usec * 1000);
@@ -1439,7 +1470,7 @@ static	int	kdoit (Timeval *timeout)
 	retval = pthread_cond_timedwait(&cond, &mutex, &right_now);
 	if (retval && retval != ETIMEDOUT)
 	{
-		syserr("kdoit(pthread): pthread_cond_timedwait() failed: %s",
+		syserr(-1, "kdoit(pthread): pthread_cond_timedwait() failed: %s",
 				strerror(retval));
 		saved_errno = retval;
 	}
@@ -1452,7 +1483,7 @@ static	int	kdoit (Timeval *timeout)
 		int c;
 
 		if ((c = pthread_join(pfd[vfd].pthread, NULL)))
-			syserr("kdoit(pthread): pthread_join(%d) failed: %s",
+			syserr(SRV(vfd), "kdoit(pthread): pthread_join(%d) failed: %s",
 					vfd, strerror(c));
 		pfd[vfd].active = 0;
 		ready++;
@@ -1472,7 +1503,7 @@ static	void	klock	(void)
 	int	c;
 
 	if ((c = pthread_mutex_lock(&mutex)))
-		panic("pthread_mutex_lock: %s", strerror(c));
+		panic(1, "pthread_mutex_lock: %s", strerror(c));
 }
 
 static	void	kunlock (void)
@@ -1480,7 +1511,7 @@ static	void	kunlock (void)
 	int	c;
 
 	if ((c = pthread_mutex_unlock(&mutex)))
-		panic("pthread_mutex_unlock: %s", strerror(c));
+		panic(1, "pthread_mutex_unlock: %s", strerror(c));
 }
 
 static	int	ksleep (double timeout)
@@ -1536,7 +1567,7 @@ static void	kinit (void)
 
 	if ((port_fd = port_create()) < 0) 
 	{
-	    syserr("kinit(ports): port_create() failed: %s", strerror(errno));
+	    syserr(-1, "kinit(ports): port_create() failed: %s", strerror(errno));
 	    irc_exit(1, "Your system doesn't support ports");
 	}
 
@@ -1550,7 +1581,7 @@ static void	ksetflag (int fd, int flag)
 	if (events[fd])
 	{
 	    if (port_dissociate(port_fd, PORT_SOURCE_FD, fd) < 0)
-		syserr("ksetflag: port_dissociate(%d) failed: %s", 
+		syserr(SRV(fd), "ksetflag: port_dissociate(%d) failed: %s", 
 			fd, strerror(errno));
 	}
 	events[fd] |= flag;
@@ -1558,7 +1589,7 @@ static void	ksetflag (int fd, int flag)
 	{
 	    if (port_associate(port_fd, PORT_SOURCE_FD, fd, 
 				events[fd], NULL) < 0)
-		syserr("ksetflag: port_associate(%d) failed: %s",
+		syserr(SRV(fd), "ksetflag: port_associate(%d) failed: %s",
 			fd, strerror(errno));
 	}
 }
@@ -1568,7 +1599,7 @@ static void	kunsetflag (int fd, int flag)
 	if (events[fd])
 	{
 	    if (port_dissociate(port_fd, PORT_SOURCE_FD, fd) < 0)
-		syserr("kunsetflag: port_dissociate(%d) failed: %s", 
+		syserr(SRV(fd), "kunsetflag: port_dissociate(%d) failed: %s", 
 			fd, strerror(errno));
 	}
 	events[fd] &= ~flag;
@@ -1576,7 +1607,7 @@ static void	kunsetflag (int fd, int flag)
 	{
 	    if (port_associate(port_fd, PORT_SOURCE_FD, fd, 
 				events[fd], NULL) < 0)
-		syserr("kunsetflag: port_associate(%d) failed: %s",
+		syserr(SRV(fd), "kunsetflag: port_associate(%d) failed: %s",
 			fd, strerror(errno));
 	}
 }
@@ -1600,13 +1631,12 @@ static	int	kdoit (Timeval *timeout)
 	struct timespec	to;
 	port_event_t	pe;
 	int		channel, retval;
-	uint_t		nget = 1;
 
 	to.tv_sec = timeout->tv_sec;
 	to.tv_nsec = timeout->tv_usec * 1000;
 
 	errno = 0;
-	retval = port_getn(port_fd, &pe, 1, &nget, &to);
+	retval = port_get(port_fd, &pe, &to);
 
 	/*
 	 * Solaris might return a negative value than -1 from port_getn()
@@ -1619,9 +1649,9 @@ static	int	kdoit (Timeval *timeout)
 		return 0;
 	}
 	else if (retval == -1 && errno != EINTR)
-		syserr("kdoit(ports): port_getn(NULL) failed: %s", 
+		syserr(-1, "kdoit(ports): port_getn(NULL) failed: %s", 
 					strerror(errno));
-	else if (retval > 0)
+	else if (retval == 0)
 	{
 		channel = pe.portev_object;
 		new_io_event(VFD(channel));
