@@ -1,4 +1,4 @@
-/* $EPIC: parse.c,v 1.84 2006/09/22 00:24:21 jnelson Exp $ */
+/* $EPIC: parse.c,v 1.89 2007/05/30 02:26:23 jnelson Exp $ */
 /*
  * parse.c: handles messages from the server.   Believe it or not.  I
  * certainly wouldn't if I were you. 
@@ -93,7 +93,7 @@ int 	is_channel (const char *to)
  * is_to_channel: determines if the argument is a channel target for
  * privmsg/notice.  STATUSMSG can appear before CHANTYPES on 005 servers.
  */
-int	is_target_channel_wall (const char *to)
+static int	is_target_channel_wall (const char *to)
 {
 	const char *	statusmsg;
 
@@ -254,6 +254,32 @@ static void	p_wallops (const char *from, const char *comm, const char **ArgList)
 
 	server_wallop = strchr(from, '.') ? 1 : 0;
 
+	/* So Black will stop asking me to add it... */
+	if (!strncmp(message, "OPERWALL - ", 11))
+	{
+		int	retval;
+
+		/* Check for ignores... */
+		if (check_ignore(from, FromUserHost, LEVEL_OPERWALL) == IGNORED)
+			return;
+
+		/* Check for floods... servers are exempted from flood checks */
+		if (!server_wallop && check_flooding(from, FromUserHost, 
+						LEVEL_OPERWALL, message))
+			return;
+
+		l = message_from(from, LEVEL_OPERWALL);
+		retval = do_hook(OPERWALL_LIST, "%s %s", from, message + 11);
+		pop_message_from(l);
+		if (!retval)
+			return;
+	}
+
+	/* 
+	 * If it's not an operwall, ,or if the user didn't catch it,
+	 * treat it as a wallop.
+	 */
+
 	/* Check for ignores... */
 	if (check_ignore(from, FromUserHost, LEVEL_WALLOP) == IGNORED)
 		return;
@@ -263,18 +289,6 @@ static void	p_wallops (const char *from, const char *comm, const char **ArgList)
 						LEVEL_WALLOP, message))
 		return;
 
-	/* So Black will stop asking me to add it... */
-	if (!strncmp(message, "OPERWALL - ", 11))
-	{
-		int	retval;
-
-		l = message_from(from, LEVEL_OPERWALL);
-		retval = do_hook(OPERWALL_LIST, "%s %s", from, message + 11);
-		pop_message_from(l);
-		if (!retval)
-			return;
-	}
-
 	l = message_from(from, LEVEL_WALLOP);
 	if (do_hook(WALLOP_LIST, "%s %c %s", 
 				from, 
@@ -482,7 +496,7 @@ static void	p_quit (const char *from, const char *comm, const char **ArgList)
 
 	if (one_prints)
 	{
-		l = message_from(what_channel(from), LEVEL_QUIT);
+		l = message_from(what_channel(from, from_server), LEVEL_QUIT);
 		if (do_hook(SIGNOFF_LIST, "%s %s", from, quit_message))
 			say("Signoff: %s (%s)", from, quit_message);
 		pop_message_from(l);
@@ -808,7 +822,8 @@ static void	p_nick (const char *from, const char *comm, const char **ArgList)
 		if (its_me)
 			l = message_from(NULL, LEVEL_NICK);
 		else
-			l = message_from(what_channel(from), LEVEL_NICK);
+			l = message_from(what_channel(from, from_server), 
+						LEVEL_NICK);
 
 		if (do_hook(NICKNAME_LIST, "%s %s", from, new_nick))
 			say("%s is now known as %s", from, new_nick);
@@ -979,7 +994,7 @@ static void	p_kick (const char *from, const char *comm, const char **ArgList)
 		     * channel we dont think we're on.
 		     */
 		    if (im_on_channel(channel, from_server))
-			panic("Window is NULL for channel [%s]", channel);
+			panic(0, "Window is NULL for channel [%s]", channel);
 
 		    yell("You were KICKed by [%s] on channel [%s] "
 			 "(reason [%s]), which you are not on!  "
