@@ -1,4 +1,4 @@
-/* $EPIC: screen.c,v 1.120 2006/10/13 21:58:02 jnelson Exp $ */
+/* $EPIC: screen.c,v 1.128 2007/07/20 22:29:33 jnelson Exp $ */
 /*
  * screen.c
  *
@@ -121,12 +121,12 @@
  *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 static void 	do_screens	(int fd);
-static int 	rite 		(Window *, const u_char *);
+static int 	rite 		(Window *, const unsigned char *);
 static void 	scroll_window   (Window *);
-static void 	add_to_window	(Window *, const u_char *);
+static void 	add_to_window	(Window *, const unsigned char *);
 static	int	ok_to_output	(Window *);
-static ssize_t read_esc_seq     (const u_char *, void *, int *);
-static ssize_t read_color_seq   (const u_char *, void *d, int);
+static ssize_t read_esc_seq     (const unsigned char *, void *, int *);
+static ssize_t read_color_seq   (const unsigned char *, void *d, int);
 
 /*
  * "Attributes" were an invention for epic5, and the general idea was
@@ -193,12 +193,12 @@ const unsigned char *all_off (void)
  * before returning.  The special case is when "old_a" is NULL, which 
  * should be treated as an explicit "all off" before handling "a".
  */
-static size_t	display_attributes (u_char *output, Attribute *old_a, Attribute *a)
+static size_t	display_attributes (unsigned char *output, Attribute *old_a, Attribute *a)
 {
-	u_char	val1 = 0x80;
-	u_char	val2 = 0x80;
-	u_char	val3 = 0x80;
-	u_char	val4 = 0x80;
+	unsigned char	val1 = 0x80;
+	unsigned char	val2 = 0x80;
+	unsigned char	val3 = 0x80;
+	unsigned char	val4 = 0x80;
 
 	if (a->reverse)		val1 |= 0x01;
 	if (a->bold)		val1 |= 0x02;
@@ -221,7 +221,7 @@ static size_t	display_attributes (u_char *output, Attribute *old_a, Attribute *a
 }
  
 /* Put into 'output', logical characters so end result is 'a' */
-static size_t	logic_attributes (u_char *output, Attribute *old_a, Attribute *a)
+static size_t	logic_attributes (unsigned char *output, Attribute *old_a, Attribute *a)
 {
 	char	*str = output;
 	size_t	count = 0;
@@ -237,8 +237,22 @@ static size_t	logic_attributes (u_char *output, Attribute *old_a, Attribute *a)
 		*str++ = ALL_OFF, count++;
 	}
 
+	if (a->reverse == 0 && a->bold == 0 && a->blink == 0 &&
+	    a->underline == 0 && a->altchar == 0 && a->fg_color == 0 &&
+	    a->bg_color == 0 && a->color_bg == 0 && a->color_fg == 0)
+	{
+	    if (old_a->reverse != 0 || old_a->bold != 0 || old_a->blink != 0 ||
+		old_a->underline != 0 || old_a->altchar != 0 || old_a->fg_color != 0 ||
+		old_a->bg_color != 0 || old_a->color_bg != 0 || old_a->color_fg != 0)
+	    {
+		*str++ = ALL_OFF;
+		*old_a = *a;
+		return 1;
+	    }
+	}
+
 	/* Colors need to be set first, always */
-	if (a->color_fg != old_a->color_fg)
+	if (a->color_fg != old_a->color_fg || a->fg_color != old_a->fg_color)
 	{
 	    *str++ = '\003', count++;
 	    if (a->color_fg)
@@ -252,7 +266,7 @@ static size_t	logic_attributes (u_char *output, Attribute *old_a, Attribute *a)
 		*str++ = '1', count++;
 	    }
 	}
-	if (a->color_bg != old_a->color_bg)
+	if (a->color_bg != old_a->color_bg || a->bg_color != old_a->bg_color)
 	{
 	    if (!a->color_fg)
 		*str++ = '\003', count++;
@@ -284,14 +298,14 @@ static size_t	logic_attributes (u_char *output, Attribute *old_a, Attribute *a)
 }
 
 /* Suppress any attribute changes in the output */
-static size_t	ignore_attributes (u_char *output, Attribute *old_a, Attribute *a)
+static size_t	ignore_attributes (unsigned char *output, Attribute *old_a, Attribute *a)
 {
 	return 0;
 }
 
 
 /* Read an attribute marker from 'input', put results in 'a'. */
-static int	read_attributes (const u_char *input, Attribute *a)
+static int	read_attributes (const unsigned char *input, Attribute *a)
 {
 	if (!input)
 		return -1;
@@ -362,7 +376,7 @@ static void	term_attribute (Attribute *a)
  *
  * DO NOT USE ANY OTHER FUNCTION TO PARSE ^C CODES.  YOU HAVE BEEN WARNED!
  */
-static ssize_t	read_color_seq (const u_char *start, void *d, int blinkbold)
+static ssize_t	read_color_seq (const unsigned char *start, void *d, int blinkbold)
 {
 	/* 
 	 * The proper "attribute" color mapping is for each ^C lvalue.
@@ -447,7 +461,7 @@ static ssize_t	read_color_seq (const u_char *start, void *d, int blinkbold)
 						  back_blink_conv;
 
 	/* Local variables, of course */
-	const 	u_char *	ptr = start;
+	const 	unsigned char *	ptr = start;
 		int		c1, c2;
 		Attribute *	a;
 		Attribute	ad;
@@ -637,14 +651,14 @@ static ssize_t	read_color_seq (const u_char *start, void *d, int blinkbold)
  *
  * DO NOT USE ANY OTHER FUNCTION TO PARSE ESCAPES.  YOU HAVE BEEN WARNED!
  */
-static ssize_t	read_esc_seq (const u_char *start, void *ptr_a, int *nd_spaces)
+static ssize_t	read_esc_seq (const unsigned char *start, void *ptr_a, int *nd_spaces)
 {
 	Attribute *	a = NULL;
 	Attribute 	safe_a;
 	int 		args[10];
 	int		nargs;
-	u_char 		chr;
-	const u_char *	str;
+	unsigned char 	chr;
+	const unsigned char *	str;
 	ssize_t		len;
 
 	if (ptr_a == NULL)
@@ -910,7 +924,7 @@ start_over:
  * State 8 is a "tab"
  * State 9 is a "non-destructive space"
  */
-static	u_char	ansi_state[256] = {
+static	unsigned char	ansi_state[256] = {
 /*	^@	^A	^B	^C	^D	^E	^F	^G(\a) */
 	6,	6,	4,	3,	6,	4,	4,	7,  /* 000 */
 /*	^H	^I	^J(\n)	^K	^L(\f)	^M(\r)	^N	^O */
@@ -1010,10 +1024,10 @@ STRIP_OTHER		X    -    -    -    -    -    -    -    X    X
 #define next_char() (*str++)
 #define put_back() (str--)
 #define nlchar '\n'
-u_char *	new_normalize_string (const u_char *str, int logical, int mangle)
+unsigned char *	new_normalize_string (const unsigned char *str, int logical, int mangle)
 {
-	u_char *	output;
-	u_char		chr;
+	unsigned char *	output;
+	unsigned char	chr;
 	Attribute	a, olda;
 	int 		pos;
 	int		maxpos;
@@ -1024,7 +1038,7 @@ u_char *	new_normalize_string (const u_char *str, int logical, int mangle)
 			strip_underline, strip_altchar, strip_color, 
 			strip_all_off, strip_nd_space, strip_c1, boldback;
 	int		strip_unprintable, strip_other;
-	size_t		(*attrout) (u_char *, Attribute *, Attribute *) = NULL;
+	size_t		(*attrout) (unsigned char *, Attribute *, Attribute *) = NULL;
 
 	mangle_escapes 	= ((mangle & MANGLE_ESCAPES) != 0);
 	normalize	= ((mangle & NORMALIZE) != 0);
@@ -1052,7 +1066,7 @@ u_char *	new_normalize_string (const u_char *str, int logical, int mangle)
 	else if (logical == 3)
 		attrout = display_attributes;	/* The status line */
 	else
-		panic("'logical == %d' is not valid.", logical);
+		panic(1, "'logical == %d' is not valid.", logical);
 
 	/* Reset all attributes to zero */
 	a.bold = a.underline = a.reverse = a.blink = a.altchar = 0;
@@ -1064,7 +1078,7 @@ u_char *	new_normalize_string (const u_char *str, int logical, int mangle)
 	 * in case you need to tack something else onto it.
 	 */
 	maxpos = strlen(str);
-	output = (u_char *)new_malloc(maxpos + 192);
+	output = (unsigned char *)new_malloc(maxpos + 192);
 	pos = 0;
 
 	while ((chr = next_char()))
@@ -1177,7 +1191,7 @@ abnormal_char:
 			    if (pos + nd_spaces > maxpos)
 			    {
 				maxpos += nd_spaces; 
-				RESIZE(output, u_char, maxpos + 192);
+				RESIZE(output, unsigned char, maxpos + 192);
 			    }
 			    while (nd_spaces-- > 0)
 			    {
@@ -1225,11 +1239,12 @@ abnormal_char:
 			str += len;
 
 			/* Suppress the color if no color is permitted */
+			if (a.bold && strip_bold)		a.bold = 0;
+			if (a.blink && strip_blink)		a.blink = 0;
 			if (strip_color)
 			{
 				a.color_fg = a.color_bg = 0;
 				a.fg_color = a.bg_color = 0;
-				break;
 			}
 
 			/* Output the new attributes */
@@ -1329,7 +1344,7 @@ abnormal_char:
 
 		default:
 		{
-			panic("Unknown normalize_string mode");
+			panic(1, "Unknown normalize_string mode");
 			break;
 		}
 	    } /* End of huge ansi-state switch */
@@ -1353,9 +1368,9 @@ abnormal_char:
  * markers) and converts them back to logical characters.  This is needed
  * for lastlog and the status line and so forth.
  */
-u_char *	denormalize_string (const u_char *str)
+unsigned char *	denormalize_string (const unsigned char *str)
 {
-	u_char *	output = NULL;
+	unsigned char *	output = NULL;
 	size_t		maxpos;
 	Attribute 	olda, a;
 	size_t		span;
@@ -1371,7 +1386,7 @@ u_char *	denormalize_string (const u_char *str)
 	 * in case you need to tack something else onto it.
 	 */
 	maxpos = strlen(str);
-	output = (u_char *)new_malloc(maxpos + 192);
+	output = (unsigned char *)new_malloc(maxpos + 192);
 	pos = 0;
 
 	while (*str)
@@ -1440,20 +1455,20 @@ static 	int 	recursion = 0,
 		line = 0,           /* Current pos in "output"      */
 		do_indent,          /* Use indent or continued line? */
 		newline = 0;        /* Number of newlines           */
-static	u_char 	**output = (unsigned char **)0;
-const 	u_char	*ptr;
-	u_char 	buffer[BIG_BUFFER_SIZE + 1],
-		c,
-		*pos_copy;
-const	u_char	*cont_ptr;
-	u_char	*cont = NULL;
-	const char *words;
+static	unsigned char 	**output = (unsigned char **)0;
+const 	unsigned char	*ptr;
+	unsigned char 	buffer[BIG_BUFFER_SIZE + 1],
+			c,
+			*pos_copy;
+const	unsigned char	*cont_ptr;
+	unsigned char	*cont = NULL;
+	const char 	*words;
 	Attribute	a, olda;
 	Attribute	saved_a;
-	u_char	*cont_free = NULL;
+	unsigned char	*cont_free = NULL;
 
 	if (recursion)
-		panic("prepare_display() called recursively");
+		panic(1, "prepare_display() called recursively");
 	recursion++;
 
         /* Reset all attributes to zero */
@@ -1467,7 +1482,7 @@ const	u_char	*cont_ptr;
 	/* do_indent = get_int_var(INDENT_VAR); */
 	do_indent = get_indent_by_winref(winref);
 	if (!(words = get_string_var(WORD_BREAK_VAR)))
-		words = ", ";
+		words = " \t";
 	if (!(cont_ptr = get_string_var(CONTINUED_LINE_VAR)))
 		cont_ptr = empty_string;
 
@@ -1842,7 +1857,7 @@ const	u_char	*cont_ptr;
  *	window		- The target window for the output
  *	str		- What is to be outputted
  */
-static int 	rite (Window *window, const u_char *str)
+static int 	rite (Window *window, const unsigned char *str)
 {
 	output_screen = window->screen;
 	scroll_window(window);
@@ -1869,12 +1884,12 @@ static int 	rite (Window *window, const u_char *str)
  * If 'output' is 1 and 'all_off' is 1, do a term_all_off() when the output
  * is done.  If 'all_off' is 0, then don't do an all_off, because
  */
-int 	output_with_count (const u_char *str1, int clreol, int output)
+int 	output_with_count (const unsigned char *str1, int clreol, int output)
 {
 	int 		beep = 0, 
 			out = 0;
 	Attribute	a;
-	const u_char *	str;
+	const unsigned char *	str;
 
         /* Reset all attributes to zero */
         a.bold = a.underline = a.reverse = a.blink = a.altchar = 0;
@@ -2006,7 +2021,7 @@ void 	add_to_screen (const unsigned char *buffer)
 	    if (is_channel(who_from))
 	    {
 		if (from_server == NOSERV)
-		    panic("Output to channel [%s:NOSERV]: %s",
+		    panic(0, "Output to channel [%s:NOSERV]: %s",
 				who_from, buffer);
 
 	        if ((tmp = get_window_by_refnum(
@@ -2137,12 +2152,12 @@ void 	add_to_screen (const unsigned char *buffer)
  * rite() handles the *appearance* of the display, writing to the screen as
  * neccesary.
  */
-static void 	add_to_window (Window *window, const u_char *str)
+static void 	add_to_window (Window *window, const unsigned char *str)
 {
 	char *		pend;
-	u_char *	strval;
-	u_char *	free_me = NULL;
-        u_char **       lines;
+	unsigned char *	strval;
+	unsigned char *	free_me = NULL;
+        unsigned char **       lines;
         int             cols;
 	int		numl = 0;
 	intmax_t	refnum;
@@ -2170,7 +2185,7 @@ static void 	add_to_window (Window *window, const u_char *str)
 	    recursion++;
 	    if (recursion < 5 && (pend = get_string_var(OUTPUT_REWRITE_VAR)))
 	    {
-		u_char	argstuff[10240];
+		unsigned char	argstuff[10240];
 
 		/* Create $* and then expand with it */
 		snprintf(argstuff, 10240, "%u %s", window->refnum, str);
@@ -2249,8 +2264,8 @@ static void 	add_to_window (Window *window, const u_char *str)
  */
 void 	add_to_window_scrollback (Window *window, const unsigned char *str, intmax_t refnum)
 {
-	u_char *	strval;
-        u_char **       lines;
+	unsigned char *	strval;
+        unsigned char **       lines;
         int             cols;
 	int		numl = 0;
 
@@ -2311,7 +2326,7 @@ static void 	scroll_window (Window *window)
 		return;
 
 	if (window->cursor > window->display_lines)
-		panic("Window [%d]'s cursor [%d] is off the display [%d]",
+		panic(1, "Window [%d]'s cursor [%d] is off the display [%d]",
 			window->refnum, window->cursor, window->display_lines);
 
 	/*
@@ -2330,7 +2345,7 @@ static void 	scroll_window (Window *window)
 		 */
 		if (window->holding_distance_from_display_ip > 
 						window->display_lines)
-			panic("Can't output to window [%d] "
+			panic(1, "Can't output to window [%d] "
 				"because it is holding stuff: [%d] [%d]", 
 				window->refnum, 
 				window->holding_distance_from_display_ip, 
@@ -2338,7 +2353,7 @@ static void 	scroll_window (Window *window)
 
 		if (window->scrollback_distance_from_display_ip > 
 						window->display_lines)
-			panic("Can't output to window [%d] "
+			panic(1, "Can't output to window [%d] "
 				"because it is scrolling back: [%d] [%d]", 
 				window->refnum, 
 				window->scrollback_distance_from_display_ip, 
@@ -2547,9 +2562,12 @@ void	create_new_screen (void)
 	new_s->quote_hit = 0;
 	new_s->fdout = 1;
 	new_s->fpout = stdout;
+#ifdef WITH_THREADED_STDOUT
+	new_s->tio_file = tio_open(new_s->fpout);
+#endif
 	new_s->fdin = 0;
 	if (use_input)
-		new_open(0, do_screens, NEWIO_READ, 0);
+		new_open(0, do_screens, NEWIO_READ, 0, -1);
 	new_s->fpin = stdin;
 	new_s->control = -1;
 	new_s->wserv_version = 0;
@@ -2561,15 +2579,13 @@ void	create_new_screen (void)
 	new_s->old_li = 0; 
 	new_s->old_co = 0;
 
-	new_s->buffer_pos = new_s->buffer_min_pos = 0;
+	new_s->buffer_pos = 0;
 	new_s->input_buffer[0] = '\0';
 	new_s->input_cursor = 0;
 	new_s->input_visible = 0;
 	new_s->input_start_zone = 0;
-	new_s->input_end_zone = 0;
-	new_s->input_prompt = NULL;
+	new_s->input_prompt = malloc_strdup(empty_string);
 	new_s->input_prompt_len = 0;
-	new_s->input_prompt_malloc = 0;
 	new_s->input_line = 23;
 
 	last_input_screen = new_s;
@@ -2645,7 +2661,7 @@ Window	*create_additional_screen (void)
 		say("Opening new window...");
 	}
 	else
-		panic("Opening new wound");
+		panic(1, "Opening new wound");
 
 	local_sockaddr.sin_family = AF_INET;
 #ifndef INADDR_LOOPBACK
@@ -2805,8 +2821,11 @@ Window	*create_additional_screen (void)
 					"to new screen");
 				return NULL;
 			}
-			new_open(new_s->fdin, do_screens, NEWIO_RECV, 1);
+			new_open(new_s->fdin, do_screens, NEWIO_RECV, 1, -1);
 			new_s->fpin = new_s->fpout = fdopen(new_s->fdin, "r+");
+#ifdef WITH_THREADED_STDOUT
+			new_s->tio_file = tio_open(new_s->fpout);
+#endif
 			continue;
 		}
 		else
@@ -2822,10 +2841,10 @@ Window	*create_additional_screen (void)
                                 return NULL;
                         }
 
-			new_open(new_s->control, do_screens, NEWIO_RECV, 1);
+			new_open(new_s->control, do_screens, NEWIO_RECV, 1, -1);
 
                         if (!(win = new_window(new_s)))
-                                panic("WINDOW is NULL and it shouldnt be!");
+                                panic(1, "WINDOW is NULL and it shouldnt be!");
                         return win;
 		}
 	    }
@@ -2863,6 +2882,10 @@ void 	kill_screen (Screen *screen)
 		add_to_invisible_list(window);
 	}
 
+#ifdef WITH_THREADED_STDOUT
+	tio_close(screen->tio_file);
+#endif
+
 	/* Take out some of the garbage left around */
 	screen->current_window = NULL;
 	screen->window_list = NULL;
@@ -3071,3 +3094,84 @@ void 	add_wait_prompt (const char *prompt, void (*func)(char *, char *), const c
 		change_input_prompt(1);
 }
 
+/*
+ * edit_char: handles each character for an input stream.  Not too difficult
+ * to work out.
+ */
+void    edit_char (unsigned char key)
+{
+        unsigned char          extended_key;
+        WaitPrompt *    oldprompt;
+        unsigned char          dummy[2];
+        int             xxx_return = 0;         /* XXXX Need i say more? */
+
+        if (dumb_mode)
+        {
+#ifdef TIOCSTI  
+                ioctl(0, TIOCSTI, &key);
+#else   
+                say("Sorry, your system doesnt support 'faking' user input...");
+#endif  
+                return;
+        }
+
+        /* were we waiting for a keypress? */
+        if (last_input_screen->promptlist && 
+                last_input_screen->promptlist->type == WAIT_PROMPT_KEY)
+        {
+                dummy[0] = key, dummy[1] = 0;
+                oldprompt = last_input_screen->promptlist;
+                last_input_screen->promptlist = oldprompt->next;
+                (*oldprompt->func)(oldprompt->data, dummy);
+                new_free(&oldprompt->data);
+                new_free(&oldprompt->prompt);
+                new_free((char **)&oldprompt);
+ 
+                set_input(empty_string);
+                change_input_prompt(-1);
+                xxx_return = 1;
+        }
+
+        /*
+         * This is only used by /pause to see when a keypress event occurs,
+         * but not to impact how that keypress is handled at all.
+         */
+        if (last_input_screen->promptlist &&
+                last_input_screen->promptlist->type == WAIT_PROMPT_DUMMY)
+        {
+                oldprompt = last_input_screen->promptlist;
+                last_input_screen->promptlist = oldprompt->next;
+                (*oldprompt->func)(oldprompt->data, NULL);
+                new_free(&oldprompt->data);
+                new_free(&oldprompt->prompt);
+                new_free((char **)&oldprompt);
+        }
+
+        if (xxx_return)
+                return;
+
+        /* If the high bit is set, mangle it as neccesary. */
+        if (key & 0x80 && current_term->TI_meta_mode)
+        {
+                edit_char('\033');
+                key &= ~0x80;
+        }
+
+        extended_key = key;
+
+        /* If we just hit the quote character, add this character literally */
+        if (last_input_screen->quote_hit)
+        {
+                last_input_screen->quote_hit = 0;
+                input_add_character(extended_key, NULL);
+        }
+
+        /* Otherwise, let the keybinding system take care of the work. */
+        else {
+                last_input_screen->last_key = handle_keypress(
+                        last_input_screen->last_key,
+                        last_input_screen->last_press, key);
+                get_time(&last_input_screen->last_press);
+        }
+}
+
