@@ -1,11 +1,11 @@
-/* $EPIC: server.c,v 1.212 2006/11/11 15:30:20 jnelson Exp $ */
+/* $EPIC: server.c,v 1.231 2007/09/07 18:07:29 jnelson Exp $ */
 /*
  * server.c:  Things dealing with that wacky program we call ircd.
  *
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1993, 2005 EPIC Software Labs.
+ * Copyright  1993, 2007 EPIC Software Labs.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -210,7 +210,7 @@ static	int	str_to_serverinfo (char *str, ServerInfo *s)
 				fieldnum = TYPE;
 			else if (!my_strnicmp(descstr, "PR", 2))
 				fieldnum = PROTO;
-			else if (!my_strnicmp(descstr, "V", 2))
+			else if (!my_strnicmp(descstr, "V", 1))
 				fieldnum = VHOST;
 			else
 			{
@@ -307,14 +307,22 @@ static	int	preserve_serverinfo (ServerInfo *si)
 	char *	resultstr = NULL;
 	size_t	clue = 0;
 
+	if (si->host && strchr(si->host, ':'))
+	   malloc_strcat_c(&resultstr, "[", &clue);
 	malloc_strcat2_c(&resultstr, si->host, ":", &clue);
+	if (si->host && strchr(si->host, ':'))
+	   malloc_strcat_c(&resultstr, "]", &clue);
 	malloc_strcat2_c(&resultstr, ltoa(si->port), ":", &clue);
 	malloc_strcat2_c(&resultstr, si->password, ":", &clue);
 	malloc_strcat2_c(&resultstr, si->nick, ":", &clue);
 	malloc_strcat2_c(&resultstr, si->group, ":", &clue);
 	malloc_strcat2_c(&resultstr, si->server_type, ":", &clue);
 	malloc_strcat2_c(&resultstr, si->proto_type, ":", &clue);
+	if (si->vhost && strchr(si->vhost, ':'))
+	   malloc_strcat_c(&resultstr, "[", &clue);
 	malloc_strcat2_c(&resultstr, si->vhost, ":", &clue);
+	if (si->vhost && strchr(si->vhost, ':'))
+	   malloc_strcat_c(&resultstr, "]", &clue);
 
 	new_free(&si->freestr);
 	new_free(&si->fulldesc);
@@ -484,6 +492,9 @@ static	int	serverinfo_to_newserv (ServerInfo *si)
 	s->umode[0] = 0;
 	s->addrs = NULL;
 	s->next_addr = NULL;
+	s->autoclose = 1;
+	s->default_realname = NULL;
+	s->realname = NULL;
 
 	s->doing_privmsg = 0;
 	s->doing_notice = 0;
@@ -1196,18 +1207,35 @@ void	do_server (int fd)
 
 #define DGETS(x, y) dgets( x , (char *) & y , sizeof y , -1);
 
+		    /* * */
+		    /* This is the errno value from getsockopt() */
 		    c = DGETS(des, retval)
 		    if (c < (ssize_t)sizeof(retval) || retval)
 			goto something_broke;
 
+		    /* This is the socket error returned by getsockopt() */
+		    c = DGETS(des, retval)
+		    if (c < (ssize_t)sizeof(retval) || retval)
+			goto something_broke;
+
+		    /* * */
+		    /* This is the errno value from getsockname() */
+		    c = DGETS(des, retval)
+		    if (c < (ssize_t)sizeof(retval) || retval)
+			goto something_broke;
+
+		    /* This is the address returned by getsockname() */
 		    c = DGETS(des, name)
 		    if (c < (ssize_t)sizeof(name))
 			goto something_broke;
 
+		    /* * */
+		    /* This is the errno value from getpeername() */
 		    c = DGETS(des, retval)
 		    if (c < (ssize_t)sizeof(retval) || retval)
 			goto something_broke;
 
+		    /* This is the address returned by getpeername() */
 		    c = DGETS(des, name)
 		    if (c < (ssize_t)sizeof(name))
 			goto something_broke;
@@ -1216,8 +1244,17 @@ void	do_server (int fd)
 		    if (0)
 		    {
 something_broke:
-			syserr("Could not connect to server [%d] address [%d]",
+			if (retval)
+			{
+			    syserr(i, "Could not connect to server [%d] "
+					"address [%d] because of error: %s", 
+					i, s->addr_counter, strerror(retval));
+			}
+			else
+			    syserr(i, "Could not connect to server [%d] "
+					"address [%d]: (Internal error)", 
 					i, s->addr_counter);
+
 			set_server_status(i, SERVER_ERROR);
 			close_server(i, NULL);
 			connect_to_server(i);
@@ -1246,7 +1283,7 @@ something_broke:
 			if (ssl_err == -1)
 			{
 			    /* XXX I don't care if this is abusive. */
-			    syserr("Could not start SSL connection to server "
+			    syserr(i, "Could not start SSL connection to server "
 				"[%d] address [%d]", 
 				i, s->addr_counter);
 			    goto something_broke;
@@ -1260,7 +1297,7 @@ something_broke:
 			 * dgets().
 			 */
 			s->status = SERVER_SSL_CONNECTING;
-			new_open(des, do_server, NEWIO_SSL_CONNECT, 0);
+			new_open(des, do_server, NEWIO_SSL_CONNECT, 0, i);
 			break;
 		    }
 
@@ -1269,12 +1306,12 @@ return_from_ssl_detour:
 		    if (is_ssl_enabled(des))
 		    {
 			set_server_ssl_enabled(i, TRUE);
-			new_open(des, do_server, NEWIO_SSL_READ, 0);
+			new_open(des, do_server, NEWIO_SSL_READ, 0, i);
 		    }
 		    else
 		    {
 			set_server_ssl_enabled(i, FALSE);
-		        new_open(des, do_server, NEWIO_RECV, 0);
+		        new_open(des, do_server, NEWIO_RECV, 0, i);
 		    }
 		    register_server(i, s->d_nickname);
 		}
@@ -1298,7 +1335,7 @@ return_from_ssl_detour:
 		    c = DGETS(des, retval)
 		    if (c < (ssize_t)sizeof(retval) || retval)
 		    {
-			syserr("SSL_connect returned [%d]", retval);
+			syserr(i, "SSL_connect returned [%d]", retval);
 			goto something_broke;
 		    }
 
@@ -1308,7 +1345,7 @@ return_from_ssl_detour:
 		     */
 		    if (ssl_connected(des) < 0)
 		    {
-			syserr("ssl_connected() failed", retval);
+			syserr(i, "ssl_connected() failed", retval);
 			goto something_broke;
 		    }
 
@@ -1519,9 +1556,10 @@ int	grab_server_address (int server)
 	set_server_status(server, SERVER_DNS);
 
 	say("Performing DNS lookup for [%s] (server %d)", s->info->host, server);
+	xvfd[0] = xvfd[1] = -1;
 	if (socketpair(PF_UNIX, SOCK_STREAM, 0, xvfd))
 		yell("socketpair: %s", strerror(errno));
-	new_open(xvfd[1], do_server, NEWIO_READ, 1);
+	new_open(xvfd[1], do_server, NEWIO_READ, 1, server);
 
 	memset(&hints, 0, sizeof(hints));
 	if (empty(s->info->proto_type))
@@ -1530,8 +1568,10 @@ int	grab_server_address (int server)
 		hints.ai_family = AF_UNSPEC;
 	else if (!my_stricmp(s->info->proto_type, "4"))
 		hints.ai_family = AF_INET;
+#ifdef INET6
 	else if (!my_stricmp(s->info->proto_type, "6"))
 		hints.ai_family = AF_INET6;
+#endif
 	else
 		hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
@@ -1561,14 +1601,14 @@ static int	connect_next_server_address (int server)
 
 	if (!(s = get_server(server)))
 	{
-		syserr("connect_next_server_address: Server %d doesn't exist", 
+		syserr(-1, "connect_next_server_address: Server %d doesn't exist", 
 						server);
 		return -1;
 	}
 
 	if (!s->addrs)
 	{
-		syserr("connect_next_server_address: There are no more "
+		syserr(server, "connect_next_server_address: There are no more "
 			"addresses available for server %d", server);
 		return -1;
 	}
@@ -1583,7 +1623,7 @@ static int	connect_next_server_address (int server)
 	    if ((err = inet_vhostsockaddr(ai->ai_family, -1, s->info->vhost,
 						&localaddr, &locallen)) < 0)
 	    {
-		syserr("connect_next_server_address: Can't use address [%d] "
+		syserr(server, "connect_next_server_address: Can't use address [%d] "
 				" because I can't get vhost for protocol [%d]",
 					 s->addr_counter, ai->ai_family);
 		continue;
@@ -1592,7 +1632,7 @@ static int	connect_next_server_address (int server)
 	    if ((fd = client_connect((SA *)&localaddr, locallen, 
 					ai->ai_addr, ai->ai_addrlen)) < 0)
 	    {
-		syserr("connect_next_server_address: "
+		syserr(server, "connect_next_server_address: "
 			"client_connect() failed for server %d address [%d].", 
 					server, s->addr_counter);
 		continue;
@@ -1696,7 +1736,7 @@ int 	connect_to_server (int new_server)
 	if (x_debug & DEBUG_SERVER_CONNECT)
 		say("connect_next_server_address returned [%d]", des);
 	from_server = new_server;	/* XXX sigh */
-	new_open(des, do_server, NEWIO_CONNECT, 0);
+	new_open(des, do_server, NEWIO_CONNECT, 0, from_server);
 
 	/* Don't check getpeername(), we're not connected yet. */
 	if (*s->info->host != '/')
@@ -1782,7 +1822,7 @@ void	close_server (int refnum, const char *message)
 	if (s->des == -1)
 		return;		/* Nothing to do here */
 
-	if (final_message && *final_message && !s->closing)
+	if (*final_message && !s->closing)
 	{
 	    s->closing = 1;
 	    if (x_debug & DEBUG_OUTBOUND)
@@ -2059,11 +2099,19 @@ void	register_server (int refnum, const char *nick)
 	if (!empty(s->info->password))
 		send_to_aserver(refnum, "PASS %s", s->info->password);
 
-	send_to_aserver(refnum, "USER %s %s %s :%s", username, 
+	malloc_strcpy (&s->realname, 
+		(s->default_realname == NULL) ?
+			(get_string_var(DEFAULT_REALNAME_VAR) ?
+		 		get_string_var(DEFAULT_REALNAME_VAR) : 
+				space) :
+		s->default_realname);
+
+	send_to_aserver(refnum, "USER %s %s %s :%s", 
+			get_string_var(DEFAULT_USERNAME_VAR),
 			(send_umode && *send_umode) ? send_umode : 
-			(LocalHostName ? LocalHostName : hostname), 
-			username, (get_string_var(REALNAME_VAR) ? 
-				   get_string_var(REALNAME_VAR) : space));
+			     (LocalHostName ? LocalHostName : hostname), 
+			get_string_var(DEFAULT_USERNAME_VAR),
+			s->realname);
 	change_server_nickname(refnum, nick);
 	if (x_debug & DEBUG_SERVER_CONNECT)
 		yell("Registered with server [%d]", refnum);
@@ -2176,9 +2224,26 @@ void  server_is_registered (int refnum, const char *itsname, const char *ourname
 	    if (new_server_lastlog_mask)
 		set_mask_by_winref(winref, *new_server_lastlog_mask);
 
+	/*
+	 * This hack is required by a race condition with freebsd that 
+	 * I'm seeing on epicsol.  For reasons that I have never been able
+	 * to adequately explain, if I write out data to the socket (ie,
+	 * from reinstate_user_modes) at the same time as the kernel is
+	 * reassembling a fractured packet (ie, the initial packet from 
+	 * the server with the 001 and stuff), the kernel will refuse to 
+	 * flag the socket as read()able ever again.  I've confirmed this
+	 * with select(), poll(), kqueue(), and even a blocking read().
+	 * Wireshark shows that the packet(s) do come in, bu the kernel
+	 * refuses to give them to me.  This tiny sleep eliminates the 
+	 * race condition that consistently causes this problem.
+	 *
+	 * P.S. -- Yes, I tried different nic cards using different drvivers.
+	 * Yes, I've tried multiple versions of freebsd.
+	 */
 	my_sleep(0.005);
+
 	reinstate_user_modes();
-	userhostbase(from_server, NULL, got_my_userhost, 1);
+	userhostbase(from_server, NULL, NULL, got_my_userhost, 1);
 
 	if (default_channel)
 	{
@@ -2248,16 +2313,22 @@ BUILT_IN_COMMAND(disconnectcmd)
 
 	if (get_server(i))
 	{
-		if (args && *args)
+		if (is_server_open(i))
+		{
+		    if (args && *args)
 			message = args;
-		else if (recon)
+		    else if (recon)
 			message = "Reconnecting";
-		else
+		    else
 			message = "Disconnecting";
 
-		say("Disconnecting from server %s", get_server_itsname(i));
-		close_server(i, message);
-		update_all_status();
+		    say("Disconnecting from server %s", get_server_itsname(i));
+		    close_server(i, message);
+		    update_all_status();
+		}
+		else if (!recon)
+		    say("You are already disconnected from server %s",
+						get_server_itsname(i));
 	}
 
 	if (!connected_to_server)
@@ -2324,7 +2395,7 @@ static SS	get_server_remote_addr (int refnum)
 	Server *s;
 
 	if (!(s = get_server(refnum)))
-	    panic("Refnum %d isn't valid in get_server_remote_addr", refnum);
+	    panic(1, "Refnum %d isn't valid in get_server_remote_addr", refnum);
 
 	return s->remote_sockname;
 }
@@ -2334,7 +2405,7 @@ SS	get_server_local_addr (int refnum)
 	Server *s;
 
 	if (!(s = get_server(refnum)))
-		panic("Refnum %d isn't valid in get_server_local_addr", refnum);
+		panic(1, "Refnum %d isn't valid in get_server_local_addr", refnum);
 
 	return s->local_sockname;
 }
@@ -2344,7 +2415,7 @@ SS	get_server_uh_addr (int refnum)
 	Server *s;
 
 	if (!(s = get_server(refnum)))
-		panic("Refnum %d isn't valid in get_server_uh_addr", refnum);
+		panic(1, "Refnum %d isn't valid in get_server_uh_addr", refnum);
 
 	return s->uh_addr;
 }
@@ -2638,6 +2709,7 @@ IACCESSOR(v, ison_len)
 IACCESSOR(v, ison_max)
 IACCESSOR(v, userhost_max)
 IACCESSOR(v, stricmp_table)
+IACCESSOR(v, autoclose)
 SACCESSOR(chan, invite_channel, NULL)
 SACCESSOR(nick, last_notify_nick, NULL)
 SACCESSOR(nick, joined_nick, NULL)
@@ -2649,6 +2721,8 @@ SACCESSOR(nick, redirect, NULL)
 SACCESSOR(message, quit_message, get_string_var(QUIT_MESSAGE_VAR))
 SACCESSOR(cookie, cookie, NULL)
 SACCESSOR(ver, version_string, NULL)
+SACCESSOR(name, default_realname, get_string_var(DEFAULT_REALNAME_VAR))
+GET_SATTRIBUTE(realname, NULL)
 
 GET_SATTRIBUTE(unique_id, NULL)
 void	set_server_unique_id (int servref, const char * id)
@@ -2701,6 +2775,19 @@ void	set_server_operator (int refnum, int flag)
 	oper_command = 0;		/* No longer doing oper */
 }
 
+static const char *	get_server_fulldesc (int servref)
+{
+	Server *s;
+
+	if (!(s = get_server(servref)))
+		return NULL;
+
+	if (s->info->fulldesc)
+		return s->info->fulldesc;
+	else
+		return NULL;
+}
+
 /* * */
 void	set_server_name (int servref, const char * param )
 {
@@ -3000,7 +3087,7 @@ static void	reset_server_altnames (int refnum, char *new_altnames)
 
 	for (i = 0; i < s->altnames->numitems; i++)
 		/* XXX Free()ing this (const char *) is ok */
-		new_free((char **)&s->altnames->list[i].name);	
+		new_free((char **)(intptr_t)&s->altnames->list[i].name);	
 
 	s->altnames->numitems = 0;
 
@@ -3122,6 +3209,37 @@ void set_server_005 (int refnum, char *setting, const char *value)
 	}
 }
 
+static char *	get_all_server_groups (void)
+{
+	Server *s;
+	int	i, j;
+	char *	retval = NULL;
+	size_t	clue;
+
+	for (i = 0; i < number_of_servers; i++)
+	{
+	    if (!(s = get_server(i)))
+		continue;
+
+	    /* 
+	     * A group is added to the return value if and only if
+	     * there is no lower server refnum of the same group.
+	     */
+	    for (j = 0; j < i; j++)
+	    {
+		if (!get_server(j))
+			continue;
+		if (!my_stricmp(get_server_group(i), get_server_group(j)))
+			goto sorry_wrong_number;
+	    }
+
+	    malloc_strcat_word_c(&retval, space, get_server_group(i), 
+					DWORD_DWORDS, &clue);
+sorry_wrong_number:
+	    ;
+	}
+	return retval;
+}
 
 /* Used by function_serverctl */
 /*
@@ -3150,11 +3268,13 @@ void set_server_005 (int refnum, char *setting, const char *value)
  *	SSL		Whether this server is SSL-enabled or not.
  *      005             Individual PROTOCTL elements.
  *      005s            The full list of PROTOCTL elements.
- *	ALIAS		An alternate server designation
- *			(SETting ALIAS adds a new alternate designation) 
- *	ALIASES		All of the alternate server designations
- *			(SETting ALIASES replaces all alternate designations)
+ *	ALTNAME		An alternate server designation
+ *			(SETting ALTNAME adds a new alternate designation) 
+ *	ALTNAMES	All of the alternate server designations
+ *			(SETting ALTNAMES replaces all alternate designations)
  *			(This is the only way to delete a designation)
+ *	DEFAULT_REALNAME Default realname, used at next connect.
+ *	REALNAME	Realname. Read-only.
  */
 char 	*serverctl 	(char *input)
 {
@@ -3178,6 +3298,8 @@ char 	*serverctl 	(char *input)
 		if (refnum != NOSERV)
 			RETURN_INT(refnum);
 		RETURN_EMPTY;
+	} else if (!my_strnicmp(listc, "ALLGROUPS", len)) {
+		RETURN_MSTR(get_all_server_groups());
 	} else if (!my_strnicmp(listc, "GET", len)) {
 		GET_INT_ARG(refnum, input);
 		if (!get_server(refnum))
@@ -3254,6 +3376,12 @@ char 	*serverctl 	(char *input)
 			int ofs = from_server;
 			char *	retval;
 
+			/* XXX Do this another way */
+			if (refnum == -1 && from_server >= 0)
+				refnum = from_server;
+			if (refnum < 0 || refnum >= number_of_servers)
+				RETURN_EMPTY;
+
 			from_server = refnum;
 			retval = get_server_005s(input);
 			from_server = ofs;
@@ -3286,6 +3414,14 @@ char 	*serverctl 	(char *input)
 			RETURN_STR(get_server_vhost(refnum));
 		} else if (!my_strnicmp(listc, "ADDRSLEFT", len)) {
 			RETURN_INT(server_addrs_left(refnum));
+		} else if (!my_strnicmp(listc, "AUTOCLOSE", len)) {
+			RETURN_INT(get_server_autoclose(refnum));
+		} else if (!my_strnicmp(listc, "FULLDESC", len)) {
+			RETURN_STR(get_server_fulldesc(refnum));
+		} else if (!my_strnicmp(listc, "REALNAME", len)) {
+			RETURN_STR(get_server_realname(refnum));
+		} else if (!my_strnicmp(listc, "DEFAULT_REALNAME", len)) {
+			RETURN_STR(get_server_default_realname(refnum));
 		}
 	} else if (!my_strnicmp(listc, "SET", len)) {
 		GET_INT_ARG(refnum, input);
@@ -3370,6 +3506,17 @@ char 	*serverctl 	(char *input)
 			add_server_altname(refnum, input);
 		} else if (!my_strnicmp(listc, "ALTNAMES", len)) {
 			reset_server_altnames(refnum, input);
+		} else if (!my_strnicmp(listc, "AUTOCLOSE", len)) {
+			int newval;
+
+			GET_INT_ARG(newval, input);
+			set_server_autoclose(refnum, newval);
+			RETURN_INT(1);
+		} else if (!my_strnicmp(listc, "REALNAME", len)) {
+			set_server_default_realname(refnum, input);
+		}
+		else if (!my_strnicmp(listc, "DEFAULT_REALNAME", len)) {
+			set_server_default_realname(refnum, input);
 		}
 	} else if (!my_strnicmp(listc, "OMATCH", len)) {
 		int	i;
