@@ -1,4 +1,4 @@
-/* $EPIC: status.c,v 1.68 2006/09/30 01:36:08 jnelson Exp $ */
+/* $EPIC: status.c,v 1.72 2007/09/14 02:55:54 jnelson Exp $ */
 /*
  * status.c: handles the status line updating, etc for IRCII 
  *
@@ -84,6 +84,7 @@ STATUS_FUNCTION(status_user);
 STATUS_FUNCTION(status_dcc);
 STATUS_FUNCTION(status_dcc_all);
 STATUS_FUNCTION(status_hold);
+STATUS_FUNCTION(status_holdmode);
 STATUS_FUNCTION(status_version);
 STATUS_FUNCTION(status_clock);
 STATUS_FUNCTION(status_hold_lines);
@@ -196,6 +197,7 @@ struct status_formats status_expandos[] = {
 { 1, '9', status_user,		NULL, 			NULL },
 { 1, 'D', status_dcc_all,	NULL, 			NULL },
 { 1, 'F', status_notify_windows,&notify_format,		&STATUS_NOTIFY_VAR },
+{ 1, 'H', status_holdmode,	NULL,			NULL },
 { 1, 'K', status_scroll_info,	NULL,			NULL },
 { 1, 'R', status_refnum_real,   NULL, 			NULL },
 { 1, 'S', status_server,        &server_format,     	&STATUS_SERVER_VAR },
@@ -503,13 +505,13 @@ int     permit_status_update (int flag)
  */
 int	make_status (Window *window, int must_redraw)
 {
-	int	status_line;
-	u_char	buffer	    [BIG_BUFFER_SIZE + 1];
-	u_char	lhs_buffer  [BIG_BUFFER_SIZE + 1];
-	u_char	rhs_buffer  [BIG_BUFFER_SIZE + 1];
-	Char	*func_value [MAX_FUNCTIONS];
-	u_char	*ptr;
-	size_t	save_size;
+	int		status_line;
+	unsigned char	buffer	    [BIG_BUFFER_SIZE + 1];
+	unsigned char	lhs_buffer  [BIG_BUFFER_SIZE + 1];
+	unsigned char	rhs_buffer  [BIG_BUFFER_SIZE + 1];
+	const char	*func_value [MAX_FUNCTIONS];
+	unsigned char	*ptr;
+	size_t		save_size;
 
 	/* We do NOT redraw status bars for hidden windows */
 	if (!window->screen || !status_updates_permitted)
@@ -517,7 +519,7 @@ int	make_status (Window *window, int must_redraw)
 
 	for (status_line = 0; status_line < window->status.number; status_line++)
 	{
-		u_char	lhs_fillchar[6],
+	unsigned char	lhs_fillchar[6],
 			rhs_fillchar[6],
 			*fillchar = lhs_fillchar,
 			*lhp = lhs_buffer,
@@ -544,7 +546,7 @@ int	make_status (Window *window, int must_redraw)
 		else if (window->status.number == 2 && status_line == 1)
 			line = 2;
 		else
-			panic("window->status.number is [%d] and status_line "
+			panic(1, "window->status.number is [%d] and status_line "
 				"is [%d] and that makes no sense!", 
 				window->status.number, status_line);
 
@@ -567,12 +569,13 @@ int	make_status (Window *window, int must_redraw)
 				return -1;
 
 			if (window->status.line[line].func[i] == NULL)
-				panic("status callback null.  Window [%d], line [%d], function [%d]", window->refnum, line, i);
+				panic(1, "status callback null.  Window [%d], line [%d], function [%d]", window->refnum, line, i);
 			func_value[i] = window->status.line[line].func[i]
 				(window, window->status.line[line].map[i],
 				 window->status.line[line].key[i]);
 		}
 
+#if 0
 		/*
 		 * If the REVERSE_STATUS_LINE var is on, then put a reverse
 		 * character in the first position (itll get translated to
@@ -581,6 +584,7 @@ int	make_status (Window *window, int must_redraw)
 		if (get_int_var(REVERSE_STATUS_LINE_VAR))
 			*buffer = REV_TOG , str = buffer + 1;
 		else
+#endif
 			str = buffer;
 
 		/*
@@ -1390,6 +1394,26 @@ STATUS_FUNCTION(status_hold)
 	return empty_string;
 }
 
+STATUS_FUNCTION(status_holdmode)
+{
+	char *text;
+
+	/* If hold mode is on... */
+	if (window->holding_top_of_display)
+	{
+	    /* ... and we're not holding anything */
+	    if (window->holding_distance_from_display_ip <
+				window->display_lines  &&
+	         window->scrollback_distance_from_display_ip <
+				window->display_lines)
+	    {
+		if ((text = get_string_var(STATUS_HOLDMODE_VAR)))
+			return text;
+	    }
+	}
+	return empty_string;
+}
+
 STATUS_FUNCTION(status_oper)
 {
 	char *text;
