@@ -1,4 +1,4 @@
-/* $EPIC: vars.c,v 1.94 2006/09/30 01:36:08 jnelson Exp $ */
+/* $EPIC: vars.c,v 1.101 2007/09/14 02:55:54 jnelson Exp $ */
 /*
  * vars.c: All the dealing of the irc variables are handled here. 
  *
@@ -144,6 +144,9 @@ IrcVariable *	clone_biv (IrcVariable *old)
 {
 	IrcVariable *var;
 
+	if (!old)
+		return NULL;		/* Take THAT! */
+
 	var = (IrcVariable *)new_malloc(sizeof(IrcVariable));
 	var->type = old->type;
 	if (old->script)
@@ -197,6 +200,9 @@ void	unclone_biv (const char *name, IrcVariable *clone)
 
 		var->flags = clone->flags;
 
+		/*
+		 * XXX This should be unified with set_variable() somehow.
+		 */
 		switch (clone->type) {
 		    case BOOL_VAR:
 		    case CHAR_VAR:
@@ -212,6 +218,30 @@ void	unclone_biv (const char *name, IrcVariable *clone)
 			break;
 		}
 
+		/* 
+		 * XXX I copied this from set_variable(), but this
+		 * should be refactored and shared with that function.
+		 */
+		if ((var->func || var->script) && !(var->flags & VIF_PENDING))
+		{
+			var->flags |= VIF_PENDING;
+			if (var->func)
+			    (var->func)(var->data);
+			if (var->script)
+			{
+			    char *s;
+			    int owd = window_display;
+
+			    s = make_string_var_bydata(var->type, (void *)var->data);
+			    window_display = 0;
+			    call_lambda_command("SET", var->script, s);
+			    window_display = owd;
+			    new_free(&s);
+			}
+			var->flags &= ~VIF_PENDING;
+		}
+
+
 		new_free(&clone->data);
 		new_free(&clone);
 		return;
@@ -267,6 +297,10 @@ void 	init_variables_stage1 (void)
 	VAR(DCC_USE_GATEWAY_ADDR, BOOL, NULL)
 #define DEFAULT_DEBUG 0
 	VAR(DEBUG, INT,  NULL);
+#define DEFAULT_DEFAULT_REALNAME NULL
+	VAR(DEFAULT_REALNAME, STR,  NULL);
+#define DEFAULT_DEFAULT_USERNAME NULL
+	VAR(DEFAULT_USERNAME, STR, NULL);
 	VAR(DISPATCH_UNKNOWN_COMMANDS, BOOL, NULL);
 	VAR(DISPLAY, BOOL, NULL);
 	VAR(DO_NOTIFY_IMMEDIATELY, BOOL, NULL);
@@ -288,6 +322,7 @@ void 	init_variables_stage1 (void)
 	VAR(KEY_INTERVAL, INT,  set_key_interval);
 	VAR(LASTLOG, INT,  set_lastlog_size);
 	VAR(LASTLOG_LEVEL, STR,  set_lastlog_mask);
+	VAR(LASTLOG_REWRITE, STR, NULL);
 #define DEFAULT_LOAD_PATH NULL
 	VAR(LOAD_PATH, STR,  NULL);
 	VAR(LOG, BOOL, logger);
@@ -324,9 +359,6 @@ void 	init_variables_stage1 (void)
 	VAR(PAD_CHAR, CHAR, NULL);
 	VAR(QUIT_MESSAGE, STR,  NULL);
 	VAR(RANDOM_SOURCE, INT,  NULL);
-#define DEFAULT_REALNAME NULL
-	VAR(REALNAME, STR,  NULL);
-	VAR(REVERSE_STATUS_LINE, BOOL, build_status);
 	VAR(SCREEN_OPTIONS, STR,  NULL);
 	VAR(SCROLLBACK, INT,  set_scrollback_size);
 	VAR(SCROLLBACK_RATIO, INT,  NULL);
@@ -350,6 +382,7 @@ void 	init_variables_stage1 (void)
 	VAR(STATUS_HALFOP, STR,  build_status);
 	VAR(STATUS_HOLD, STR,  build_status);
 	VAR(STATUS_HOLD_LINES, STR,  build_status);
+	VAR(STATUS_HOLDMODE, STR,  build_status);
 	VAR(STATUS_INSERT, STR,  build_status);
 	VAR(STATUS_MAIL, STR,  build_status);
 	VAR(STATUS_MODE, STR,  build_status);
@@ -802,7 +835,7 @@ char 	*make_string_var_bydata (int type, void *vp)
 			ret = malloc_dupchar(data->integer);
 			break;
 		default:
-			panic("make_string_var_bydata: unrecognized type [%d]", type);
+			panic(1, "make_string_var_bydata: unrecognized type [%d]", type);
 	}
 	return (ret);
 
@@ -1022,6 +1055,8 @@ void    set_indent (void *stuff)
 
 
 /***************************************************************************/
+/* XXX Apparantly this is dead code. */
+#if 0
 /*******/
 typedef struct	varstacklist
 {
@@ -1117,5 +1152,6 @@ void	do_stack_set (int type, char *args)
 	else
 		say("Unknown STACK type ??");
 }
+#endif
 
 
